Package: simulateDCE
Title: Simulate Data for Discrete Choice Experiments
Version: 0.3.2
Authors@R: 
    person(given = "Julian", family = "Sagebiel", email = "julian.sagebiel@idiv.de", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0253-6875"))
Description: Supports simulating choice experiment data for given designs. It helps to quickly test different designs against each other and compare the performance of new models. The goal of 'simulateDCE' is to make it easy to simulate choice experiment datasets using designs from 'NGENE', 'idefix' or 'spdesign'. You have to store the design file(s) in a sub-directory and need to specify certain parameters and the utility functions for the data generating process. For more details on choice experiments see Mariel et al. (2021) <doi:10.1007/978-3-030-62669-3>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: dplyr (>= 1.1.4), evd, formula.tools, ggplot2, kableExtra,
        magrittr, mixl, psych, purrr, readr, rmarkdown, stats, utils,
        stringr, tibble, tictoc, tidyr, future, furrr, qs2, data.table,
        glue
Suggests: knitr, testthat (>= 3.0.0), rlang
Config/testthat/edition: 3
Depends: R (>= 4.1.0)
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-03 22:54:42 UTC; dj44vuri
Author: Julian Sagebiel [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0253-6875>)
Maintainer: Julian Sagebiel <julian.sagebiel@idiv.de>
Repository: CRAN
Date/Publication: 2026-05-04 07:00:11 UTC
Built: R 4.5.2; ; 2026-05-04 07:50:40 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(aggregateResults)
export(createDataset)
export(extract_b_values)
export(readdesign)
export(sim_all)
export(sim_choice)
export(simulate_choices)
import(data.table)
importFrom(dplyr,arrange)
importFrom(dplyr,mutate)
importFrom(dplyr,relocate)
importFrom(dplyr,row_number)
importFrom(dplyr,slice)
importFrom(magrittr,"%>%")

<!-- README.md is generated from README.Rmd. Please edit that file -->

# simulateDCE

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/simulateDCE)](https://CRAN.R-project.org/package=simulateDCE)

[![R-CMD-check](https://github.com/sagebiej/simulatedce/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/sagebiej/simulatedce/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

The goal of simulateDCE is to make it easy to simulate choice experiment
datasets using designs from NGENE, `idefix` or `spdesign`. You have to
store the design file(s) in a subfolder and need to specify certain
parameters and the utility functions for the data generating process.
The package is useful for:

1.  Test different designs in terms of statistical power, efficiency and
    unbiasedness

2.  To test the effects of deviations from RUM, e.g. heuristics, on
    model performance for different designs.

3.  In teaching, using simulated data is useful, if you want to know the
    data generating process. It helps to demonstrate Maximum likelihood
    and choice models, knowing exactly what you should expect.

4.  You can use simulation in pre-registration to justify your sample
    size and design choice.

5.  Before data collection, you can use simulated data to estimate the
    models you plan to use in the actual analysis. You can thus make
    sure, you can estimate all effects for given sample sizes.

## Installation

You can install simulateDCE directly from cran by

``` r
install.packages("simulateDCE") 
```

For the latest development version use this:

``` r
install.packages("devtools")
devtools::install_git('https://github.com/sagebiej/simulateDCE', ref = "devel")
```

## Example

This is a basic example for a simulation:

``` r
rm(list = ls())
library(simulateDCE)
library(rlang)
library(formula.tools)
#> 
#> Attaching package: 'formula.tools'
#> The following object is masked from 'package:rlang':
#> 
#>     env


designpath <- system.file("extdata", "SE_DRIVE", package = "simulateDCE")

resps <- 120 # number of respondents
nosim <- 4 # number of simulations to run (about 500 is minimum)


decisiongroups <- c(0, 0.7, 1)

# place b coefficients into an r list:
bcoeff <- list(
  bpreis = -0.01,
  blade = -0.07,
  bwarte = 0.02
)

manipulations <- list(
  alt1.x2 = expr(alt1.x2 / 10),
  alt1.x3 = expr(alt1.x3 / 10),
  alt2.x2 = expr(alt2.x2 / 10),
  alt2.x3 = expr(alt2.x3 / 10)
)


# place your utility functions here
ul <- list(
  u1 =

    list(
      v1 = V.1 ~ bpreis * alt1.x1 + blade * alt1.x2 + bwarte * alt1.x3,
      v2 = V.2 ~ bpreis * alt2.x1 + blade * alt2.x2 + bwarte * alt2.x3
    ),
  u2 = list(
    v1 = V.1 ~ bpreis * alt1.x1,
    v2 = V.2 ~ bpreis * alt2.x1
  )
)


destype <- "ngene"

sedrive <- sim_all(
  nosim = nosim, resps = resps, destype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups
)
#> 'simple' is deprecated and will be removed in the future. Use 'exact' instead.
#> bcoeff_lookup already exists; skipping modification.
#> 
#> Utility function used in simulation (true utility):
#> $u1
#> $u1$v1
#> V.1 ~ bpreis * alt1.x1 + blade * alt1.x2 + bwarte * alt1.x3
#> <environment: 0x5fd289ec7eb8>
#> 
#> $u1$v2
#> V.2 ~ bpreis * alt2.x1 + blade * alt2.x2 + bwarte * alt2.x3
#> <environment: 0x5fd289e68760>
#> 
#> 
#> $u2
#> $u2$v1
#> V.1 ~ bpreis * alt1.x1
#> <environment: 0x5fd289dccb80>
#> 
#> $u2$v2
#> V.2 ~ bpreis * alt2.x1
#> <environment: 0x5fd289d6a748>
#> 'destype' is deprecated. Please use 'designtype' instead.
#> New names:
#> • `Choice situation` -> `Choice.situation`
#> • `` -> `...10`
#> assign keys for bcoeff): 0 sec elapsed
#> user entered manipulations: 0.01 sec elapsed
#> split dataframe into groups: 0.005 sec elapsed
#> for each group calculate utility: 0.891 sec elapsed
#> add random component: 0.058 sec elapsed
#> whole simulate choices: 0.974 sec elapsed
#> 
#>  No preprocess function provided. Proceeding without additional preprocessing.
#> 
#> 
#>  dataset preprossed_data exists: FALSE
#> 
#> 
#>  decisiongroups exists: TRUE
#> 
#> Group counts:
#> 
#>    1    2 
#> 1007  433 
#> 
#>  data has been created 
#> 
#> 
#> First few observations of the dataset
#>   ID Choice_situation alt1_x1 alt1_x2 alt1_x3 alt2_x1 alt2_x2 alt2_x3 Block
#> 1  1                7      80      25     100      60     200     100     1
#> 2  1               19      20      25      50      60      25       0     1
#> 3  1               30      20     100      50      80      50     100     1
#> 4  1               32      40     200      25      80      25       0     1
#> 5  1               39      40     200       0      80     100     100     1
#> 6  1               48      60      50      25      20      50     100     1
#>   group    V_1    V_2        e_1          e_2         U_1         U_2 CHOICE
#> 1     1  -0.55 -12.60  0.7017091 -0.004845669   0.1517091 -12.6048457      1
#> 2     1  -0.95  -2.35 -0.7570334 -0.314662810  -1.7070334  -2.6646628      1
#> 3     1  -6.20  -2.30  0.5284701  0.765330842  -5.6715299  -1.5346692      2
#> 4     1 -13.90  -2.55  0.6972737  1.276552182 -13.2027263  -1.2734478      2
#> 5     1 -14.40  -5.80  2.3723773  3.840028537 -12.0276227  -1.9599715      2
#> 6     1  -3.60  -1.70  0.7219891  0.900484573  -2.8780109  -0.7995154      2
#> assign keys for bcoeff): 0 sec elapsed
#> user entered manipulations: 0.012 sec elapsed
#> split dataframe into groups: 0.004 sec elapsed
#> for each group calculate utility: 1.17 sec elapsed
#> add random component: 0.063 sec elapsed
#> whole simulate choices: 1.259 sec elapsed
#> 
#>  No preprocess function provided. Proceeding without additional preprocessing.
#> 
#> 
#>  dataset preprossed_data exists: FALSE
#> 
#> 
#>  decisiongroups exists: TRUE
#> 
#> Group counts:
#> 
#>    1    2 
#> 1007  433 
#> 
#>  data has been created 
#> 
#> 
#> First few observations of the dataset
#>   ID Choice_situation alt1_x1 alt1_x2 alt1_x3 alt2_x1 alt2_x2 alt2_x3 Block
#> 1  1                7      80      25     100      60     200     100     1
#> 2  1               19      20      25      50      60      25       0     1
#> 3  1               30      20     100      50      80      50     100     1
#> 4  1               32      40     200      25      80      25       0     1
#> 5  1               39      40     200       0      80     100     100     1
#> 6  1               48      60      50      25      20      50     100     1
#>   group    V_1    V_2         e_1         e_2         U_1        U_2 CHOICE
#> 1     1  -0.55 -12.60  2.75514992 -0.66469500   2.2051499 -13.264695      1
#> 2     1  -0.95  -2.35  0.00533537 -0.04035054  -0.9446646  -2.390351      1
#> 3     1  -6.20  -2.30  0.74855512  0.39861844  -5.4514449  -1.901382      2
#> 4     1 -13.90  -2.55  0.23703861  0.51413370 -13.6629614  -2.035866      2
#> 5     1 -14.40  -5.80  1.63619258 -0.12188201 -12.7638074  -5.921882      2
#> 6     1  -3.60  -1.70 -0.32856943 -1.04084037  -3.9285694  -2.740840      2
#> assign keys for bcoeff): 0 sec elapsed
#> user entered manipulations: 0.011 sec elapsed
#> split dataframe into groups: 0.005 sec elapsed
#> for each group calculate utility: 1.115 sec elapsed
#> add random component: 0.053 sec elapsed
#> whole simulate choices: 1.195 sec elapsed
#> 
#>  No preprocess function provided. Proceeding without additional preprocessing.
#> 
#> 
#>  dataset preprossed_data exists: FALSE
#> 
#> 
#>  decisiongroups exists: TRUE
#> 
#> Group counts:
#> 
#>    1    2 
#> 1007  433 
#> 
#>  data has been created 
#> 
#> 
#> First few observations of the dataset
#>   ID Choice_situation alt1_x1 alt1_x2 alt1_x3 alt2_x1 alt2_x2 alt2_x3 Block
#> 1  1                7      80      25     100      60     200     100     1
#> 2  1               19      20      25      50      60      25       0     1
#> 3  1               30      20     100      50      80      50     100     1
#> 4  1               32      40     200      25      80      25       0     1
#> 5  1               39      40     200       0      80     100     100     1
#> 6  1               48      60      50      25      20      50     100     1
#>   group    V_1    V_2        e_1          e_2         U_1        U_2 CHOICE
#> 1     1  -0.55 -12.60 -0.1655165  0.333722053  -0.7155165 -12.266278      1
#> 2     1  -0.95  -2.35 -0.5967960  0.047598095  -1.5467960  -2.302402      1
#> 3     1  -6.20  -2.30  0.1815357  0.151706904  -6.0184643  -2.148293      2
#> 4     1 -13.90  -2.55  1.0162094 -0.003073446 -12.8837906  -2.553073      2
#> 5     1 -14.40  -5.80  1.9017193 -1.307206385 -12.4982807  -7.107206      2
#> 6     1  -3.60  -1.70  3.0272325  0.101946794  -0.5727675  -1.598053      1
#> assign keys for bcoeff): 0 sec elapsed
#> user entered manipulations: 0.016 sec elapsed
#> split dataframe into groups: 0.007 sec elapsed
#> for each group calculate utility: 0.871 sec elapsed
#> add random component: 0.041 sec elapsed
#> whole simulate choices: 0.949 sec elapsed
#> 
#>  No preprocess function provided. Proceeding without additional preprocessing.
#> 
#> 
#>  dataset preprossed_data exists: FALSE
#> 
#> 
#>  decisiongroups exists: TRUE
#> 
#> Group counts:
#> 
#>    1    2 
#> 1007  433 
#> 
#>  data has been created 
#> 
#> 
#> First few observations of the dataset
#>   ID Choice_situation alt1_x1 alt1_x2 alt1_x3 alt2_x1 alt2_x2 alt2_x3 Block
#> 1  1                7      80      25     100      60     200     100     1
#> 2  1               19      20      25      50      60      25       0     1
#> 3  1               30      20     100      50      80      50     100     1
#> 4  1               32      40     200      25      80      25       0     1
#> 5  1               39      40     200       0      80     100     100     1
#> 6  1               48      60      50      25      20      50     100     1
#>   group    V_1    V_2        e_1        e_2         U_1         U_2 CHOICE
#> 1     1  -0.55 -12.60  1.6738036 -1.5621745   1.1238036 -14.1621745      1
#> 2     1  -0.95  -2.35  1.4617051  2.8739225   0.5117051   0.5239225      2
#> 3     1  -6.20  -2.30 -1.1041301  1.1893797  -7.3041301  -1.1106203      2
#> 4     1 -13.90  -2.55 -0.1083358 -0.6587746 -14.0083358  -3.2087746      2
#> 5     1 -14.40  -5.80  0.8077831 -0.7007500 -13.5922169  -6.5007500      2
#> 6     1  -3.60  -1.70  1.5490572  0.3438886  -2.0509428  -1.3561114      2
#> 
#> 
#> 
#> Transformed utility function (type: simple):
#> [1] "U_1 = @bpreis * $alt1_x1 + @blade * $alt1_x2 + @bwarte * $alt1_x3 ;U_2 = @bpreis * $alt2_x1 + @blade * $alt2_x2 + @bwarte * $alt2_x3 ;"
#> Initial function value: -998.1319 
#> Initial gradient value:
#> bpreis  blade bwarte 
#>   6360 -39600   7025 
#> initial  value 998.131940 
#> iter   2 value 800.504025
#> iter   3 value 786.395435
#> iter   4 value 784.242446
#> iter   5 value 750.719727
#> iter   6 value 742.063100
#> iter   7 value 740.790227
#> iter   8 value 740.763095
#> iter   9 value 740.763059
#> iter   9 value 740.763050
#> iter   9 value 740.763044
#> final  value 740.763044 
#> converged
#> Initial function value: -998.1319 
#> Initial gradient value:
#> bpreis  blade bwarte 
#>   6200 -37900   7650 
#> initial  value 998.131940 
#> iter   2 value 821.888191
#> iter   3 value 809.241040
#> iter   4 value 808.473296
#> iter   5 value 769.609504
#> iter   6 value 761.231597
#> iter   7 value 759.946167
#> iter   8 value 759.922557
#> iter   9 value 759.922524
#> iter   9 value 759.922513
#> iter   9 value 759.922506
#> final  value 759.922506 
#> converged
#> Initial function value: -998.1319 
#> Initial gradient value:
#> bpreis  blade bwarte 
#>   6160 -38625   6950 
#> initial  value 998.131940 
#> iter   2 value 815.623636
#> iter   3 value 802.757357
#> iter   4 value 800.768673
#> iter   5 value 764.276430
#> iter   6 value 755.754107
#> iter   7 value 754.402132
#> iter   8 value 754.373606
#> iter   9 value 754.373562
#> iter   9 value 754.373551
#> iter   9 value 754.373545
#> final  value 754.373545 
#> converged
#> Initial function value: -998.1319 
#> Initial gradient value:
#> bpreis  blade bwarte 
#>   6620 -38900   6025 
#> initial  value 998.131940 
#> iter   2 value 815.994858
#> iter   3 value 806.153552
#> iter   4 value 804.395555
#> iter   5 value 768.518125
#> iter   6 value 759.936308
#> iter   7 value 758.360035
#> iter   8 value 758.320822
#> iter   9 value 758.320725
#> iter  10 value 758.320713
#> iter  10 value 758.320710
#> iter  10 value 758.320705
#> final  value 758.320705 
#> converged
#> start_estimation: 0.212 sec elapsed
#> 
#> Summary table:
#> 
#> 
#> ================  ====  ===  =====  ====  ======  =====  =====  =====  ====  ========  ====
#> \                 vars    n   mean    sd  median    min    max  range  skew  kurtosis    se
#> ================  ====  ===  =====  ====  ======  =====  =====  =====  ====  ========  ====
#> est_bpreis           1    4  -0.01  0.00   -0.01  -0.01   0.00   0.00  0.38     -2.01  0.00
#> est_blade            2    4  -0.01  0.00   -0.01  -0.02  -0.01   0.00  0.11     -1.89  0.00
#> est_bwarte           3    4   0.01  0.00    0.01   0.01   0.01   0.00  0.15     -1.88  0.00
#> rob_pval0_bpreis     4    4   0.05  0.04    0.03   0.02   0.11   0.09  0.69     -1.72  0.02
#> rob_pval0_blade      5    4   0.00  0.00    0.00   0.00   0.00   0.00  0.75     -1.69  0.00
#> rob_pval0_bwarte     6    4   0.00  0.00    0.00   0.00   0.00   0.00  0.75     -1.69  0.00
#> ================  ====  ===  =====  ====  ======  =====  =====  =====  ====  ========  ====
#> 
#> Power results:
#> 
#> FALSE  TRUE 
#>    25    75 
#> 'simple' is deprecated and will be removed in the future. Use 'exact' instead.
#> bcoeff_lookup already exists; skipping modification.
#> 
#> Utility function used in simulation (true utility):
#> $u1
#> $u1$v1
#> V.1 ~ bpreis * alt1.x1 + blade * alt1.x2 + bwarte * alt1.x3
#> <environment: 0x5fd28c1b5a60>
#> 
#> $u1$v2
#> V.2 ~ bpreis * alt2.x1 + blade * alt2.x2 + bwarte * alt2.x3
#> <environment: 0x5fd28c199d60>
#> 
#> 
#> $u2
#> $u2$v1
#> V.1 ~ bpreis * alt1.x1
#> <environment: 0x5fd28c16e850>
#> 
#> $u2$v2
#> V.2 ~ bpreis * alt2.x1
#> <environment: 0x5fd28c14bb58>
#> 
#> 
#> 'destype' is deprecated. Please use 'designtype' instead.
#> New names:
#> assign keys for bcoeff): 0.001 sec elapsed
#> user entered manipulations: 0.011 sec elapsed
#> split dataframe into groups: 0.006 sec elapsed
#> for each group calculate utility: 0.966 sec elapsed
#> add random component: 0.057 sec elapsed
#> whole simulate choices: 1.053 sec elapsed
#> 
#>  No preprocess function provided. Proceeding without additional preprocessing.
#> 
#> 
#>  dataset preprossed_data exists: FALSE
#> 
#> 
#>  decisiongroups exists: TRUE
#> 
#> Group counts:
#> 
#>    1    2 
#> 1007  433 
#> 
#>  data has been created 
#> 
#> 
#> First few observations of the dataset
#>   ID Choice_situation alt1_x1 alt1_x2 alt1_x3 alt2_x1 alt2_x2 alt2_x3 Block
#> 1  1               12      60      25       0      20     200     100     1
#> 2  1               16      20     100      50      40      50       0     1
#> 3  1               17      20     200       0      80     100     100     1
#> 4  1               25      60      50     100      20     200      50     1
#> 5  1               29      20      50     100      80      50       0     1
#> 6  1               32      40     100      25      80      25      50     1
#>   group    V_1    V_2        e_1        e_2        U_1        U_2 CHOICE
#> 1     1  -2.35 -12.20  0.7348289  2.8760442  -1.615171  -9.323956      1
#> 2     1  -6.20  -3.90 -0.4325068 -1.1036864  -6.632507  -5.003686      2
#> 3     1 -14.20  -5.80 -0.6013954 -0.6508939 -14.801395  -6.450894      2
#> 4     1  -2.10 -13.20 -0.7730537  2.5897688  -2.873054 -10.610231      1
#> 5     1  -1.70  -4.30 -0.8569517  2.3587195  -2.556952  -1.941280      2
#> 6     1  -6.90  -1.55  1.9317262 -1.4418899  -4.968274  -2.991890      2
#> assign keys for bcoeff): 0 sec elapsed
#> user entered manipulations: 0.012 sec elapsed
#> split dataframe into groups: 0.005 sec elapsed
#> for each group calculate utility: 1.133 sec elapsed
#> add random component: 0.056 sec elapsed
#> whole simulate choices: 1.217 sec elapsed
#> 
#>  No preprocess function provided. Proceeding without additional preprocessing.
#> 
#> 
#>  dataset preprossed_data exists: FALSE
#> 
#> 
#>  decisiongroups exists: TRUE
#> 
#> Group counts:
#> 
#>    1    2 
#> 1007  433 
#> 
#>  data has been created 
#> 
#> 
#> First few observations of the dataset
#>   ID Choice_situation alt1_x1 alt1_x2 alt1_x3 alt2_x1 alt2_x2 alt2_x3 Block
#> 1  1               12      60      25       0      20     200     100     1
#> 2  1               16      20     100      50      40      50       0     1
#> 3  1               17      20     200       0      80     100     100     1
#> 4  1               25      60      50     100      20     200      50     1
#> 5  1               29      20      50     100      80      50       0     1
#> 6  1               32      40     100      25      80      25      50     1
#>   group    V_1    V_2        e_1         e_2         U_1        U_2 CHOICE
#> 1     1  -2.35 -12.20 -1.3863937  0.01166719  -3.7363937 -12.188333      1
#> 2     1  -6.20  -3.90 -0.7563038  0.81627154  -6.9563038  -3.083728      2
#> 3     1 -14.20  -5.80 -0.8547753  0.83640581 -15.0547753  -4.963594      2
#> 4     1  -2.10 -13.20  0.1376063 -0.20383194  -1.9623937 -13.403832      1
#> 5     1  -1.70  -4.30  0.8143902  0.31880374  -0.8856098  -3.981196      1
#> 6     1  -6.90  -1.55  0.7225187 -0.56312651  -6.1774813  -2.113127      2
#> assign keys for bcoeff): 0 sec elapsed
#> user entered manipulations: 0.011 sec elapsed
#> split dataframe into groups: 0.007 sec elapsed
#> for each group calculate utility: 1.184 sec elapsed
#> add random component: 0.052 sec elapsed
#> whole simulate choices: 1.266 sec elapsed
#> 
#>  No preprocess function provided. Proceeding without additional preprocessing.
#> 
#> 
#>  dataset preprossed_data exists: FALSE
#> 
#> 
#>  decisiongroups exists: TRUE
#> 
#> Group counts:
#> 
#>    1    2 
#> 1007  433 
#> 
#>  data has been created 
#> 
#> 
#> First few observations of the dataset
#>   ID Choice_situation alt1_x1 alt1_x2 alt1_x3 alt2_x1 alt2_x2 alt2_x3 Block
#> 1  1               12      60      25       0      20     200     100     1
#> 2  1               16      20     100      50      40      50       0     1
#> 3  1               17      20     200       0      80     100     100     1
#> 4  1               25      60      50     100      20     200      50     1
#> 5  1               29      20      50     100      80      50       0     1
#> 6  1               32      40     100      25      80      25      50     1
#>   group    V_1    V_2         e_1        e_2         U_1        U_2 CHOICE
#> 1     1  -2.35 -12.20  0.06710733  0.1285654  -2.2828927 -12.071435      1
#> 2     1  -6.20  -3.90 -1.32075517  2.0607289  -7.5207552  -1.839271      2
#> 3     1 -14.20  -5.80 -0.72258548 -0.1480184 -14.9225855  -5.948018      2
#> 4     1  -2.10 -13.20  2.27096778  1.1434050   0.1709678 -12.056595      1
#> 5     1  -1.70  -4.30  0.91051706 -0.4674794  -0.7894829  -4.767479      1
#> 6     1  -6.90  -1.55  1.53679884  3.0082291  -5.3632012   1.458229      2
#> assign keys for bcoeff): 0 sec elapsed
#> user entered manipulations: 0.009 sec elapsed
#> split dataframe into groups: 0.003 sec elapsed
#> for each group calculate utility: 1.052 sec elapsed
#> add random component: 0.044 sec elapsed
#> whole simulate choices: 1.118 sec elapsed
#> 
#>  No preprocess function provided. Proceeding without additional preprocessing.
#> 
#> 
#>  dataset preprossed_data exists: FALSE
#> 
#> 
#>  decisiongroups exists: TRUE
#> 
#> Group counts:
#> 
#>    1    2 
#> 1007  433 
#> 
#>  data has been created 
#> 
#> 
#> First few observations of the dataset
#>   ID Choice_situation alt1_x1 alt1_x2 alt1_x3 alt2_x1 alt2_x2 alt2_x3 Block
#> 1  1               12      60      25       0      20     200     100     1
#> 2  1               16      20     100      50      40      50       0     1
#> 3  1               17      20     200       0      80     100     100     1
#> 4  1               25      60      50     100      20     200      50     1
#> 5  1               29      20      50     100      80      50       0     1
#> 6  1               32      40     100      25      80      25      50     1
#>   group    V_1    V_2         e_1       e_2        U_1        U_2 CHOICE
#> 1     1  -2.35 -12.20 -1.28989396 0.9484000 -3.6398940 -11.251600      1
#> 2     1  -6.20  -3.90  0.74918818 0.1249138 -5.4508118  -3.775086      2
#> 3     1 -14.20  -5.80  5.69606611 1.1375340 -8.5039339  -4.662466      2
#> 4     1  -2.10 -13.20  2.90516321 0.2937650  0.8051632 -12.906235      1
#> 5     1  -1.70  -4.30  2.77751745 1.5906729  1.0775174  -2.709327      1
#> 6     1  -6.90  -1.55 -0.01728414 3.3229082 -6.9172841   1.772908      2
#> 
#> 
#> 
#> Transformed utility function (type: simple):
#> [1] "U_1 = @bpreis * $alt1_x1 + @blade * $alt1_x2 + @bwarte * $alt1_x3 ;U_2 = @bpreis * $alt2_x1 + @blade * $alt2_x2 + @bwarte * $alt2_x3 ;"
#> Initial function value: -998.1319 
#> Initial gradient value:
#> bpreis  blade bwarte 
#>   5360 -37200   7650 
#> initial  value 998.131940 
#> iter   2 value 842.455891
#> iter   3 value 830.557321
#> iter   4 value 830.235796
#> iter   5 value 790.231947
#> iter   6 value 781.816563
#> iter   7 value 780.429591
#> iter   8 value 780.381041
#> iter   9 value 780.380266
#> iter  10 value 780.380038
#> iter  11 value 780.379732
#> iter  12 value 780.379533
#> iter  12 value 780.379533
#> iter  12 value 780.379533
#> final  value 780.379533 
#> converged
#> Initial function value: -998.1319 
#> Initial gradient value:
#> bpreis  blade bwarte 
#>   4100 -37475   7400 
#> initial  value 998.131940 
#> iter   2 value 839.617713
#> iter   3 value 834.663927
#> iter   4 value 829.106636
#> iter   5 value 778.893518
#> iter   6 value 769.219181
#> iter   7 value 767.652744
#> iter   8 value 767.626805
#> iter   9 value 767.626779
#> iter  10 value 767.626763
#> iter  10 value 767.626761
#> iter  10 value 767.626757
#> final  value 767.626757 
#> converged
#> Initial function value: -998.1319 
#> Initial gradient value:
#> bpreis  blade bwarte 
#>   4760 -37675   8175 
#> initial  value 998.131940 
#> iter   2 value 832.730149
#> iter   3 value 832.267432
#> iter   4 value 830.145207
#> iter   5 value 778.690334
#> iter   6 value 769.072881
#> iter   7 value 767.565134
#> iter   8 value 767.540015
#> iter   9 value 767.539640
#> iter  10 value 767.539493
#> iter  11 value 767.539455
#> iter  12 value 767.539374
#> iter  12 value 767.539374
#> iter  12 value 767.539374
#> final  value 767.539374 
#> converged
#> Initial function value: -998.1319 
#> Initial gradient value:
#> bpreis  blade bwarte 
#>   4840 -37450   8225 
#> initial  value 998.131940 
#> iter   2 value 835.537197
#> iter   3 value 821.095140
#> iter   4 value 820.186712
#> iter   5 value 780.436158
#> iter   6 value 772.106098
#> iter   7 value 770.910821
#> iter   8 value 770.861700
#> iter   9 value 770.860779
#> iter  10 value 770.860582
#> iter  11 value 770.860346
#> iter  12 value 770.860168
#> iter  12 value 770.860168
#> iter  12 value 770.860168
#> final  value 770.860168 
#> converged
#> start_estimation: 0.128 sec elapsed
#> 
#> Summary table:
#> 
#> 
#> ================  ====  ===  =====  ====  ======  =====  =====  =====  =====  ========  ====
#> \                 vars    n   mean    sd  median    min    max  range   skew  kurtosis    se
#> ================  ====  ===  =====  ====  ======  =====  =====  =====  =====  ========  ====
#> est_bpreis           1    4  -0.01  0.00   -0.01  -0.01   0.00   0.01  -0.37     -1.85  0.00
#> est_blade            2    4  -0.01  0.00   -0.01  -0.01  -0.01   0.00  -0.17     -1.92  0.00
#> est_bwarte           3    4   0.01  0.00    0.01   0.00   0.01   0.00  -0.73     -1.70  0.00
#> rob_pval0_bpreis     4    4   0.02  0.03    0.00   0.00   0.07   0.07   0.75     -1.69  0.02
#> rob_pval0_blade      5    4   0.00  0.00    0.00   0.00   0.00   0.00   0.66     -1.74  0.00
#> rob_pval0_bwarte     6    4   0.00  0.00    0.00   0.00   0.00   0.00   0.75     -1.69  0.00
#> ================  ====  ===  =====  ====  ======  =====  =====  =====  =====  ========  ====
#> 
#> Power results:
#> 
#> FALSE  TRUE 
#>    25    75
#> total time for simulation and estimation: 18.547 sec elapsed
#> $tic
#> elapsed 
#>   1.131 
#> 
#> $toc
#> elapsed 
#>  19.678 
#> 
#> $msg
#> [1] "total time for simulation and estimation"
#> 
#> $callback_msg
#> [1] "total time for simulation and estimation: 18.547 sec elapsed"
#> 
#> Trying tidyr::pivot_longer for reshaping...
```

<img src="man/figures/README-example-1.png" width="100%" /><img src="man/figures/README-example-2.png" width="100%" /><img src="man/figures/README-example-3.png" width="100%" />

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(simulateDCE)
library(rlang)
library(formula.tools)

## ----initialize---------------------------------------------------------------
bcoeff <- list(
  basc = 4.2, ## very high asc
  bprof = 0.3,
  bexp = 0.3,
  bdomestic = 0.3,
  bforeign = 0.3,
  bdamage = 0.6,
  bprice = 0.2
)



manipulations <- list(
  alt1.professional = expr(alt1.initiator == 1),
  alt2.professional = expr(alt2.initiator == 1),
  alt1.expert = expr(alt1.initiator == 2),
  alt2.expert = expr(alt2.initiator == 2),
  alt1.domestic = expr(alt1.funding == 1),
  alt2.domestic = expr(alt2.funding == 1),
  alt1.foreign = expr(alt1.funding == 2),
  alt2.foreign = expr(alt2.funding == 2)
)


# place your utility functions here
ul <- list(
  u1 =
    list(
      v1 = V.1 ~ bprof * alt1.professional + bexp * alt1.expert + bdomestic * alt1.domestic + bforeign * alt1.foreign + bdamage * alt1.damage + bprice * alt1.compensation,
      v2 = V.2 ~ bprof * alt2.professional + bexp * alt2.expert + bdomestic * alt2.domestic + bforeign * alt2.foreign + bdamage * alt2.damage + bprice * alt2.compensation,
      v3 = V.3 ~ basc
    )
)

## ----other--------------------------------------------------------------------
designpath <- system.file("extdata", "SE_AGRI", package = "simulateDCE")
## can also be specified using relative path eg. designpath<- "Projects/CSA/Designs/"

# notes <- "This design consists of different heuristics. One group did not attend the methan attribute, another group only decided based on the payment"

notes <- "No Heuristics"

resps <- 240 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

## design type must be either 'spdesign' or 'ngene'
destype <- "ngene"

## ----random-------------------------------------------------------------------
set.seed(3393)

## ----output-------------------------------------------------------------------
seagri <- simulateDCE::sim_all(
  nosim = nosim, resps = resps, designtype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, manipulations = manipulations
)

## ----accessOutput-------------------------------------------------------------
designs <- names(seagri[sapply(seagri, is.list)])

print(designs)

simulationCoeff <- seagri[[1]]$coefs
coeffSummary <- seagri[[1]]$summary

print(simulationCoeff)
print(coeffSummary)


## saveRDS(csa,file = "tests/manual-tests/csa.RDS")

# Taken from Erlend Sandorf
# Example file for creating a simple MNL design
#
rm(list = ls(all = TRUE))
library(spdesign)

# Define the list of utility functions ----
#' Specifying a utility function with 3 attributes and a constant for the
#' SQ alternative. The design has 20 rows.
utility <- list(
  alt1 = "b_x1[0.1]  * x1[1:5] + b_x2[0.4] * x2[c(0, 1)] ",
  alt2 = "b_x1       * x1      + b_x2      * x2          "
)

# Generate designs ----
design <- generate_design(utility, rows = 20,
                          model = "mnl", efficiency_criteria = "d-error",
                          algorithm = "rsc", draws = "scrambled-sobol",
                          control = list(
                            max_iter = 2000,
                            max_no_improve = 5000
                          ))

saveRDS(design,"inst/extdata/spdesigns/designs/twoattr.RDS")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(simulateDCE)
library(rlang)
library(formula.tools)

## ----initialize---------------------------------------------------------------
bcoeff <- list(
  bx1 = -0.1,
  bx2 = -0.1,
  bx3 = -0.05,
  bx4 = -0.025
)

# place your utility functions here
ul <- list(u1 = list(
  v1 = V.1 ~ bx1 * alt1.x1 + bx2 * alt1.x2 + bx3 * alt1.x3 + bx4 * alt1.x4,
  v2 = V.2 ~ bx1 * alt2.x1 + bx2 * alt2.x2 + bx3 * alt2.x3 + bx4 * alt2.x4,
  v3 = V.3 ~ -5
))

## ----other--------------------------------------------------------------------
designpath <- system.file("extdata", "CSA", "linear", package = "simulateDCE")
## can also be specified using relative path eg. designpath<- "Projects/CSA/Designs/"

# notes <- "This design consists of different heuristics. One group did not attend the methan attribute, another group only decided based on the payment"

notes <- "No Heuristics"

resps <- 240 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

## design type must be either 'spdesign' or 'ngene'
destype <- "spdesign"

## ----random-------------------------------------------------------------------
set.seed(3393)

## ----output-------------------------------------------------------------------
csa <- simulateDCE::sim_all(
  nosim = nosim, resps = resps, designtype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff
)

## ----accessOutput-------------------------------------------------------------
topLevelResults <- names(csa[sapply(csa, is.list)])

print(topLevelResults)

## saves and prints the key results of the first expreimental design
simulationCoeff <- csa[[1]]$coefs
coeffSummary <- csa[[1]]$summary

print(simulationCoeff)
print(coeffSummary)
## saveRDS(csa,file = "tests/manual-tests/csa.RDS")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(simulateDCE)
library(rlang)
library(formula.tools)

## ----initialize---------------------------------------------------------------
# pass beta coefficients as a list
bcoeff <- list(
  bpreis = -0.01,
  blade = -0.07,
  bwarte = 0.02
)

manipulations <- list(
  alt1.x2 = expr(alt1.x2 / 10),
  alt1.x3 = expr(alt1.x3 / 10),
  alt2.x2 = expr(alt2.x2 / 10),
  alt2.x3 = expr(alt2.x3 / 10)
)


# place your utility functions here
ul <- list(
  u1 =

    list(
      v1 = V.1 ~ bpreis * alt1.x1 + blade * alt1.x2 + bwarte * alt1.x3,
      v2 = V.2 ~ bpreis * alt2.x1 + blade * alt2.x2 + bwarte * alt2.x3
    ),
  u2 = list(
    v1 = V.1 ~ bpreis * alt1.x1,
    v2 = V.2 ~ bpreis * alt2.x1
  )
)

## ----decision-----------------------------------------------------------------
decisiongroups <- c(0, 0.7, 1)

## ----other--------------------------------------------------------------------
designpath <- system.file("extdata", "SE_DRIVE", package = "simulateDCE")

resps <- 120 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)


destype <- "ngene"

## ----output-------------------------------------------------------------------
sedrive <- sim_all(
  nosim = nosim, resps = resps, designtype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups
)

## ----accessOutput-------------------------------------------------------------
## nested results are hard coded, if the design changes this must aswell
simulationCoeff <- sedrive$olddesign$coefs
coeffSummary <- sedrive$olddesign$summary

print(simulationCoeff)
print(coeffSummary)

#' Simulate and estimate choices
#'
#' @param designfile path to a file containing a design.
#' @param no_sim Number of runs i.e. how often do you want the simulation to be repeated
#' @param respondents Number of respondents. How many respondents do you want to simulate in each run.
#' @param u A list with utility functions. The list can incorporate as many decision rule groups as you want. However, each group must be in a list in this list. If you just use one group (the normal),  this  group still  has to be in a list in  the u list. As a convention name beta coefficients starting with a lower case "b"
#' @param estimate If TRUE models will be estimated. If false only a dataset will be simulated. Default is TRUE
#' @inheritParams readdesign
#' @inheritParams simulate_choices
#' @param chunks The number of chunks determines how often results should be stored on disk as a safety measure to not loose simulations if models have already been estimated. For example, if no_sim is 100 and chunks = 2, the data will be saved on disk after 50 and after 100 runs.
#' @param utility_transform_type How the utility function you entered is transformed to the utility function required for mixl. You can use the classic way (simple) where parameters have to start with "b" and variables with "alt" or the more flexible (but potentially error prone) way (exact) where parameters and variables are matched exactly what how the are called in the dataset and in the bcoeff list. Default is "simple". In the long run, simple will be deleted, as exact should be downwards compatible.
#' @param mode Set to "parallel" if parts should be run in parallel mode
#' @param savefile Indicate a path if you want to store the results after each design simulation locally. This is useful in case you fear that your computer crashes
#' @return a list with all information on the run
#' @export
#'
#' @examples bcoeff <- list(
#'   basc = -1.2,
#'   basc2 = -1.4,
#'   baction = 0.1,
#'   badvisory = 0.4,
#'   bpartnertest = 0.3,
#'   bcomp = 0.02
#' )
#' ul <- list(
#'   u1 =
#'     list(
#'      #' # model specification ----------------------------------------------
#' v1 <- V.1 ~ basc +
#'   baction      * alt1.b +
#'   badvisory    * alt1.c +
#'   bpartnertest * alt1.d +
#'   bcomp        * alt1.p,
#'
#' v2 <- V.2 ~ basc2 +
#'   baction      * alt2.b +
#'   badvisory    * alt2.c +
#'   bpartnertest * alt2.d +
#'   bcomp        * alt2.p,
#'  v3 <- V.3 ~ 0
#'     )
#' )
#'
#' sim_choice(
#'   designfile = system.file("extdata", "agora", "altscf_eff.ngd", package = "simulateDCE"),
#'   no_sim = 2,
#'   respondents = 144,
#'   u = ul,
#'   bcoeff = bcoeff,
#'   estimate = FALSE
#' )
#'
sim_choice <- function(designfile, no_sim = 10, respondents = 330, u,
                       designtype = NULL, destype = NULL, bcoeff,
                       decisiongroups = c(0, 1), manipulations = list(), estimate, chunks = 1,
                       utility_transform_type = "simple", mode = c("parallel", "sequential"),
                       preprocess_function = NULL,
                       savefile = NULL) {
  mode <- match.arg(mode)

  #################################################
  ########## Input Validation Test ###############
  #################################################

  # Stop condition to check if chunks is a positive integer
  if (!is.numeric(chunks) || chunks <= 0 || chunks != as.integer(chunks)) {
    stop("`chunks` must be a positive integer.")
  }

  if (utility_transform_type == "simple") {
    message("'simple' is deprecated and will be removed in the future. Use 'exact' instead.")
  }



  ## make bcoeff clean
  bcoeff_result <- modify_bcoeff_names(bcoeff)
  bcoeff <- bcoeff_result$bcoeff
  bcoeff_lookup <- bcoeff_result$bcoeff_lookup

  ### make utility function clean
  u <- purrr::map(u, function(utility_group) {
    purrr::map(utility_group, function(utility) {
      # Convert the RHS of the formula to a single string
      rhs_string <- paste(deparse(formula.tools::rhs(utility)), collapse = " ")

      # Replace coefficient names in the RHS string
      modified_rhs <- stringr::str_replace_all(
        rhs_string,
        stats::setNames(
          bcoeff_lookup$modified,
          paste0("(?<![a-zA-Z0-9._])", bcoeff_lookup$original, "(?![a-zA-Z0-9._])")
        )
      )

      # Recreate the formula with the modified RHS
      stats::formula(paste(as.character(formula.tools::lhs(utility)), "~", modified_rhs))
    })
  })


  ### function to store results
  savef <- function(object) {
    if (!is.null(savefile)) {
      # Create the directory if it does not exist
      save_dir <- dirname(savefile)
      if (!dir.exists(save_dir)) {
        dir.create(save_dir, recursive = TRUE)
        message("Directory created: ", save_dir)
      }
      qs2::qs_save(object, paste0(savefile, "_", basename(designname), ".qs"))
      message("Output saved to: ", paste0(savefile, "_", basename(designname), ".qs"))
    }
  }



  # Empty list where to store all designs later on
  designs_all <- list()

  #### Print some messages ####

  ## one-liner ---------------------------------------------------------------
  message(
    "\nUtility function used in simulation (true utility):\n",
    paste(utils::capture.output(print(u)), collapse = "\n")
  )








  #### Read in the design file and set core variables ####



  design <- readdesign(design = designfile, designtype = designtype, destype = destype)


  dname <- designname <- stringr::str_remove_all(designfile, "(.ngd|_|.RDS)")

  datadet <- simulateDCE::createDataset(design = design, respondents = respondents)


  switchmap <- function(.x, .f, mode, workers = NULL, ..., .progress = TRUE) {
    switch(mode,
      "parallel" = {
        # Set up parallel backend
        if (!is.null(workers)) {
          future::plan("multisession", workers = workers)
        } else {
          future::plan("multisession")
        }
        on.exit(future::plan("sequential"), add = TRUE) # Ensure plan is reset after execution
        furrr::future_map(.x, .f, ..., .options = furrr::furrr_options(seed = TRUE))
      },
      "sequential" = {
        purrr::map(.x, .f, ..., .progress = .progress)
      }
    )
  }


  sim_data <- 1:no_sim %>% switchmap(~ simulate_choices(datadet, utility = u, bcoeff = bcoeff, decisiongroups = decisiongroups, manipulations = manipulations, preprocess_function = preprocess_function), mode = mode)


  ### start estimation

  if (estimate == TRUE) {
    database <- sim_data[[1]]


    ####  Function that transforms user written utility for simulation into utility function for mixl.
    transform_util <- function() {
      mnl_U <- paste(purrr::map_chr(u[[1]], as.character, keep.source.attr = TRUE), collapse = "", ";") %>%
        stringr::str_replace_all(c("priors\\[\"" = "", "\"\\]" = "", "~" = "=", "\\." = "_", " b" = " @b", "V_" = "U_", " alt" = " $alt"))
    }


    transform_util2 <- function() {
      # Exclude columns that match "V_<any integer>" or "U_<any integer>" pattern
      relevant_database_vars <- setdiff(
        names(database),
        c(grep("^(V_|U_|e_)\\d+$", names(database), value = TRUE), "CHOICE")
      )

      mnl_U <- paste(
        purrr::map_chr(u[[1]], as.character, keep.source.attr = TRUE),
        collapse = "",
        ";"
      ) %>%
        # Replace coefficients with exact matches
        stringr::str_replace_all(stats::setNames(
          paste0("@", names(bcoeff)),
          paste0("(?<![._a-zA-Z0-9])", names(bcoeff), "(?![._a-zA-Z0-9-])")
        )) %>%
        # General transformations
        stringr::str_replace_all(c(
          `priors\\["` = "",
          `"\\]` = "",
          `~` = "=",
          `\\.` = "_", ## can be deleted when everything works
          `V_` = "U_" ## was originally V_
        )) %>%
        # Replace only relevant database variables
        stringr::str_replace_all(stats::setNames(
          paste0("$", relevant_database_vars),
          paste0("(?<![._a-zA-Z0-9])", relevant_database_vars, "(?![._a-zA-Z0-9-])")
        )) %>%
        # Clean up duplicate symbols
        stringr::str_replace_all(c(`@@` = "@", "\\$\\$" = "$"))

      return(mnl_U)
    }




    # transform utility function to mixl format

    # transform utility function to mixl format
    mnl_U <- switch(utility_transform_type,
      "simple" = transform_util(),
      "exact" = transform_util2(),
      stop("Invalid utility_transform_type. Use 'simple' or 'exact'.")
    )

    ## message-based version ---------------------------------------------------
    message(
      "\nTransformed utility function (type: ", utility_transform_type, "):\n",
      paste(utils::capture.output(print(mnl_U)), collapse = "\n")
    )



    # specify model for mixl estimation

    model_spec <- mixl::specify_model(utility_script = mnl_U, dataset = database, disable_multicore = T)

    est <- stats::setNames(rep(0, length(model_spec$beta_names)), model_spec$beta_names)


    availabilities <- mixl::generate_default_availabilities(
      database, model_spec$num_utility_functions
    )

    if (chunks > 1) {
      # Calculate the size of each chunk
      chunk_size <- ceiling(no_sim / chunks)

      # Initialize the starting point for the first chunk
      start_point <- 1

      for (i in seq_along(1:chunks)) {
        # Calculate the end point for the current chunk
        end_point <- start_point + chunk_size - 1

        # Ensure we do not go beyond the total number of simulations
        if (end_point > no_sim) {
          end_point <- no_sim
        }

        # Run estimations for the current chunk

        tictoc::tic(paste0("start_estimation of chunk ", i))

        output <- start_point:end_point %>%
          purrr::map(
            ~ mixl::estimate(
              model_spec = model_spec,
              start_values = est,
              availabilities = availabilities,
              data = sim_data[[.x]]
            )
          )
        message( utils::capture.output(tictoc::toc(log = FALSE, quiet = TRUE)) )

        chunkfilename <- paste0(dname, "_tmp_", i, ".qs")

        qs2::qs_save(output, chunkfilename)
        rm(output)

        gc()

        ## message-based progress note --------------------------------------------
        message(sprintf("Results for chunk %s from %s to %s", i, start_point, end_point))


        # Update the start point for the next chunk
        start_point <- end_point + 1

        # Break the loop if the end point reaches or exceeds no_sim
        if (start_point > no_sim) break
      }


      output <- list() # Initialize the list to store all outputs

      # Assuming the files are named in sequence as 'tmp_1.RDS', 'tmp_2.RDS', ..., 'tmp_n.RDS'
      for (i in seq_along(1:chunks)) {
        # Load each RDS file
        chunkfilename <- paste0(dname, "_tmp_", i, ".qs")
        file_content <- qs2::qs_read(chunkfilename)
        file.remove(chunkfilename)

        # Append the contents of each file to the all_outputs list
        output <- c(output, file_content)
      }
    } else {
      tictoc::tic("start_estimation")
      output <- purrr::map(
        sim_data,
        ~ mixl::estimate(
          model_spec = model_spec,
          start_values = est,
          availabilities = availabilities,
          data = .x
        )
      )
      message( utils::capture.output(tictoc::toc(log = FALSE, quiet = TRUE)) )
    }



    coefs <- purrr::map(1:length(output), ~ summary(output[[.]])[["coefTable"]][c(1, 8)] %>%
      tibble::rownames_to_column() %>%
      tidyr::pivot_wider(names_from = rowname, values_from = c(est, rob_pval0))) %>%
      dplyr::bind_rows(.id = "run")

    output[["summary"]] <- psych::describe(coefs[, -1], fast = TRUE)

    output[["coefs"]] <- coefs

    pvals <- output[["coefs"]] %>% dplyr::select(dplyr::starts_with("rob_pval0"))

    output[["power"]] <- 100 * table(apply(pvals, 1, function(x) all(x < 0.05))) / nrow(pvals)


    output[["metainfo"]] <- c(Path = designfile, NoSim = no_sim, NoResp = respondents)


    ## ----- summary table -----------------------------------------------------
    message(
      "\nSummary table:\n",
      paste(
        utils::capture.output(
          print(kableExtra::kable(output[["summary"]], digits = 2, format = "rst"))
        ),
        collapse = "\n"
      )
    )

    ## ----- power results -----------------------------------------------------
    message(
      "\nPower results:\n",
      paste(
        utils::capture.output(print(output[["power"]])),
        collapse = "\n"
      )
    )






    savef(output)
    return(output)
  } else { # if estimate not TRUE, return only simulated data.
    savef(sim_data)
    return(sim_data)
  }
}

#' Create a Dataset for Choice Experiment Analysis
#'
#' This function takes a design matrix and generates a dataset for use in choice experiments.
#' It handles blocks, replicates the design for the number of respondents, and assigns respondent IDs.
#'
#' @param design A data frame containing the design matrix for the choice experiment.
#'   It should include at least the columns `Choice.situation` and optionally `Block`.
#'
#' @param respondents The number of respondents to generate data for.
#'
#' @details
#' The function performs the following steps:
#' \itemize{
#'   \item Checks if the `Block` column exists in the input design. If absent, it creates a single block.
#'   \item Calculates the number of choice sets and blocks, and determines the number of sets per block.
#'   \item Replicates the design to account for the specified number of respondents per block.
#'   \item Assigns respondent IDs based on the number of respondents and blocks.
#' }
#'
#' @return A data frame containing the augmented design matrix with additional columns:
#' \describe{
#'   \item{ID}{A unique identifier for each respondent.}
#'   \item{Choice.situation}{The original choice situations, replicated for respondents.}
#'   \item{Other columns}{All original columns in the input `design` are retained.}
#' }
#'
#' @importFrom dplyr arrange slice row_number mutate relocate
#' @export
#'
#' @examples
#' # Example usage:
#' design <- data.frame(
#'   Choice.situation = rep(1:12),
#'   Attribute1 = rnorm(12),
#'   Attribute2 = sample(1:3, 12, replace = TRUE)
#' )
#' result <- createDataset(design, 10)
createDataset <- function(design, respondents) {
  if (!("Block" %in% colnames(design))) design$Block <- 1 # If no Blocks exist, create a variable Blocks to indicate it is only one block

  nsets <- nrow(design)
  nblocks <- max(design$Block)
  setpp <- nsets / nblocks # Choice Sets per respondent

  replications <- respondents / nblocks

  # assign("nsets", nsets, envir = parent.frame())
  # assign("nblocks", nblocks, envir = parent.frame())
  # assign("setpp", setpp, envir = parent.frame())
  # assign("replications", replications, envir = parent.frame())

  ## if replications is non int, assign unevenly

  datadet <- design %>%
    dplyr::arrange(Block, Choice.situation) %>%
    dplyr::slice(rep(dplyr::row_number(), replications)) %>% ## replicate design according to number of replications

    dplyr::mutate(ID = rep(1:respondents, each = setpp)) %>% # create Respondent ID.
    dplyr::relocate(ID, `Choice.situation`) %>%
    as.data.frame()
}

library(rlang)

designpath <- system.file("extdata", "Rbook", package = "simulateDCE")

design <- system.file("extdata", "Rbook", "design1.RDS", package = "simulateDCE")

# notes <- "This design consists of different heuristics. One group did not attend the methan attribute, another group only decided based on the payment"

notes <- "No Heuristics"

resps <- 40 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

# betacoefficients should not include "-"

bcoeff <- list(
  bsq = 0.00,
  bredkite = -0.05,
  bdistance = 0.50,
  bcost = -0.05,
  bfarm2 = 0.25,
  bfarm3 = 0.50,
  bheight2 = 0.25,
  bheight3 = 0.50
)

destype <- "spdesign"


# place your utility functions here
ul <- list(u1 = list(
  v1 = V.1 ~ bsq * alt1.sq,
  v2 = V.2 ~ bfarm2 * alt2.farm2 + bfarm3 * alt2.farm3 + bheight2 * alt2.height2 + bheight3 * alt2.height3 + bredkite * alt2.redkite + bdistance * alt2.distance + bcost * alt2.cost,
  v3 = V.3 ~ bfarm2 * alt3.farm2 + bfarm3 * alt3.farm3 + bheight2 * alt3.height2 + bheight3 * alt3.height3 + bredkite * alt3.redkite + bdistance * alt3.distance + bcost * alt3.cost
))


formattedes <- readdesign(design = design, designtype = "spdesign")
data <- simulateDCE::createDataset(formattedes, respondents = resps)


test_that("simulate_choices() warns when deprecated setspp is used", {
  expect_warning(
    simulate_choices(
      data   = data,
      utility = ul,
      setspp = 4,
      bcoeff = bcoeff
    ),
    regexp = "is deprecated and ignored"
  )
})

test_that("simulate_choices() does not error", {
  expect_error(
    simulate_choices(data = data, bcoeff = bcoeff, u = ul),
    regexp = NA  # ← this is key: expect *no* error
  )
})

ds <- simulate_choices(data = data, bcoeff = bcoeff, u = ul)
test_that("random values are unique", {
  expect_equal(dim(table(table(ds$e_1))), 1)
  expect_equal(dim(table(table(ds$e_2))), 1)
  expect_equal(dim(table(table(ds$e_3))), 1)
  expect_equal(dim(table(table(ds$U_1))), 1)
  expect_equal(dim(table(table(ds$U_2))), 1)
  expect_equal(dim(table(table(ds$U_3))), 1)
  expect_true("CHOICE" %in% names(ds))
  expect_equal(length(unique(ds$CHOICE)), 3)
  expect_equal(length(unique(ds$Block)), 10)
  expect_equal(length(unique(ds$Choice_situation)), 100)
  expect_equal(length(unique(ds$ID)), 40)
  expect_equal(length(unique(ds$group)), 1)
})


#### more simple tests

# assume your simulate_choices() is already loaded

#–– 1) small toy dataset & simple utility for testing ––#
df_small <- data.frame(
  ID      = rep(1:5, each = 2),
  price   = rep(c(10, 20), 5),
  quality = rep(c(1,  2), 5)
)

beta <- list(
  bprice   = -1,
  bquality =  2
)

# note: use V.1 / V.2 so that rename_with("\\.", "_") => V_1/V_2
ut <- list(
  u1 = list(
    v1 = V.1 ~ bprice * price + bquality * quality,
    v2 = V.2 ~ 0
  )
)

#––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––#

test_that("basic output: columns, types, and deterministic V", {
  res <- simulate_choices(
    data = df_small,
    utility = ut,
    bcoeff = beta
  )

  # must be a data.frame
  expect_s3_class(res, "data.frame")

  # must contain the original columns + group + V_*/e_*/U_*/CHOICE
  expect_true(all(
    c("price","quality","V_1","V_2","e_1","e_2","U_1","U_2","CHOICE")
    %in% names(res)
  ))

  # since V_1 = -1*price + 2*quality, V_2 = 0
  # and these are created *before* the random error is added,
  # they must match exactly:
  expect_equal(res$V_1, -1 * res$price + 2 * res$quality)
  expect_equal(res$V_2, rep(0, nrow(res)))
})

test_that("CHOICE always in {1,2} for two alternatives", {
  res <- simulate_choices(df_small, ut, bcoeff = beta)
  expect_true(all(res$CHOICE %in% c(1L, 2L)))
})

test_that("preprocess_function must be a function", {
  expect_error(
    simulate_choices(df_small, ut,  bcoeff = beta,
                     preprocess_function = 123),
    "`preprocess_function` must be a function"
  )
})

test_that("preprocess_function merges back on ID", {
  # create a df with explicit ID
  df2 <- df_small
  df2$ID <- 1:nrow(df2)

  # preprocessing returns only ID=1 with extra column
  prep <- function() data.frame(ID = 1, extra = 99)

  res <- simulate_choices(df2, ut, bcoeff = beta,
                          preprocess_function = prep)

  expect_true("extra" %in% names(res))
  expect_equal(res$extra[1], 99)
  expect_true(all(is.na(res$extra[-1])))
})

test_that("manipulations are applied before utility", {
  # e.g. triple the price
  manip <- list(price = expr(price * 3))

  res <- simulate_choices(df_small, ut,
                          bcoeff = beta, manipulations = manip)

  # price in result should be three times the original
  expect_equal(res$price, df_small$price * 3)

  # and V_1 = -1 * (price*3) + 2*quality
  expect_equal(res$V_1, -1 * res$price + 2 * res$quality)
})

test_that("decisiongroups produces correct group labels", {
  # 10 rows → break at 50% → two groups of 5 each

  ut2 <- list(
    u1 = list(
      v1 = V.1 ~ bprice * price + bquality * quality,
      v2 = V.2 ~ 0
    ),
    u2 = list(
      v1 = V.1 ~ bprice * price + bquality * quality * 1.5,
      v2 = V.2 ~ 0
    )
  )
  dg   <- c(0, .5, 1)
  df10 <- df_small[1:10, ]
  df10$ID <- 1:10

  res <- simulate_choices(df10, ut2,
                          bcoeff = beta, decisiongroups = dg)

  # first 5 rows group==1, next 5 group==2
  expect_equal(res$group, rep(1:2, each = 5))
})

#––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––#


### MORE GROUP SPECIFIC TESTS


test_that("utilities are applied correctly by group", {
  # 10 rows, alternating prices and qualities
  df10 <- data.frame(
    ID      = 1:10,
    price   = rep(c(10, 20), 5),
    quality = rep(c(1,  2), 5)
  )

  beta <- list(
    bprice   = -1,
    bquality =  2
  )

  ut <- list(
    g1 = list(
      v1 = V.1 ~ bprice * price + bquality * quality,  # group 1
      v2 = V.2 ~ 0
    ),
    g2 = list(
      v1 = V.1 ~ bprice * price + 2 * bquality * quality,  # group 2 → stronger quality
      v2 = V.2 ~ 0
    )
  )

  res <- simulate_choices(
    df10,
    utility = ut,
    bcoeff  = beta,
    decisiongroups = c(0, 0.5, 1)  # → 2 groups of 5
  )

  # extract utilities
  res1 <- res[res$group == 1, ]
  res2 <- res[res$group == 2, ]

  # expected: V_1 = -1 * price + (2 * quality)
  v1_g1_expected <- with(res1, -1 * price + 2 * quality)
  expect_equal(res1$V_1, v1_g1_expected)

  # group 2 has double quality weight: V_1 = -1 * price + 4 * quality
  v1_g2_expected <- with(res2, -1 * price + 4 * quality)
  expect_equal(res2$V_1, v1_g2_expected)
})


test_that("missing utility group throws error", {
  df10 <- data.frame(
    ID = 1:10,
    price   = rep(c(10, 20), 5),
    quality = rep(c(1,  2), 5)
  )

  beta <- list(bprice = -1, bquality = 2)

  ut <- list(only_one = list(
    v1 = V.1 ~ bprice * price + bquality * quality,
    v2 = V.2 ~ 0
  ))

  expect_error(
    simulate_choices(df10, ut,  bcoeff = beta,
                     decisiongroups = c(0, 0.5, 1)),
    "Length of `utility`.*does not match.*decision groups"
  )
})


test_that("group in data not covered by utility triggers error", {
  df10 <- data.frame(
    ID = 1:10,
    price   = rep(c(10, 20), 5),
    quality = rep(c(1,  2), 5)
  )

  beta <- list(bprice = -1, bquality = 2)

  ut <- list(
    g1 = list(
      v1 = V.1 ~ bprice * price + bquality * quality,
      v2 = V.2 ~ 0
    ),
    g2 = list(
      v1 = V.1 ~ bprice * price + bquality * quality,
      v2 = V.2 ~ 0
    )
  )

  # corrupt the decision group vector
  expect_error(
    simulate_choices(df10, ut, bcoeff = beta,
                     decisiongroups = c(0, 0.4, 0.8, 1)),  # 3 breaks but only 2 utils
    "Length of `utility`.*does not match.*decision groups"
  )
})

rm(list = ls())
devtools::load_all()

designpath <- system.file("extdata", "feedadditives", package = "simulateDCE")

# notes <- "This design consists of different heuristics. One group did not attend the methan attribute, another group only decided based on the payment"

notes <- "Three heuristics"

resps <- 396 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)
destype <- "ngene"

# betacoefficients should not include "-"
bcoeff <- list(
  basc = 0.2,
  bcow = 0.3,
  badv = 0.3,
  bvet = 0.3,
  bfar = 0.3,
  bmet = 0.3,
  bbon = 0.3,
  bbon2 = 1.9,
  basc2 = 2
)

decisiongroups <- c(0, 0.3, 0.6, 0.8, 1)

# place your utility functions here
ul <- list(
  u1 = list(
    v1 = V.1 ~ bcow * alt1.cow + badv * alt1.adv + bvet * alt1.vet + bfar * alt1.far + bmet * alt1.met + bbon * alt1.bon,
    v2 = V.2 ~ bcow * alt2.cow + badv * alt2.adv + bvet * alt2.vet + bfar * alt2.far + bmet * alt2.met + bbon * alt2.bon,
    v3 = V.3 ~ basc
  ),
  u2 = list(
    v1 = V.1 ~ bcow * alt1.cow + badv * alt1.adv + bvet * alt1.vet + bfar * alt1.far + bbon * alt1.bon,
    v2 = V.2 ~ bcow * alt2.cow + badv * alt2.adv + bvet * alt2.vet + bfar * alt2.far + bbon * alt2.bon,
    v3 = V.3 ~ basc
  ),
  u3 = list(
    v1 = V.1 ~ bbon2 * alt1.bon,
    v2 = V.2 ~ bbon2 * alt2.bon,
    v3 = V.3 ~ basc
  ),
  u4 = list(
    v1 = V.1 ~ basc2 + bcow * alt1.cow + badv * alt1.adv + bvet * alt1.vet + bfar * alt1.far + bmet * alt1.met + bbon * alt1.bon,
    v2 = V.2 ~ bcow * alt2.cow + badv * alt2.adv + bvet * alt2.vet + bfar * alt2.far + bmet * alt2.met + bbon * alt2.bon,
    v3 = V.3 ~ basc
  )
)

feedadditives <- sim_all(
  nosim = nosim, resps = resps, designtype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups
)

rm(list = ls())
devtools::load_all()
library(rlang)


designpath <- system.file("extdata", "SE_DRIVE", package = "simulateDCE")

resps <- 120 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

decisiongroups <- c(0, 0.7, 1)

# pass beta coefficients as a list
bcoeff <- list(
  b.preis = -0.01,
  b.lade = -0.07,
  b.warte = 0.02
)

manipulations <- list(
  alt1.x2 = expr(alt1.x2 / 10),
  alt1.x3 = expr(alt1.x3 / 10),
  alt2.x2 = expr(alt2.x2 / 10),
  alt2.x3 = expr(alt2.x3 / 10)
)


# place your utility functions here
ul <- list(
  u1 =

    list(
      v1 = V.1 ~ b.preis * alt1.x1 + b.lade * alt1.x2 + b.warte * alt1.x3,
      v2 = V.2 ~ b.preis * alt2.x1 + b.lade * alt2.x2 + b.warte * alt2.x3
    ),
  u2 = list(
    v1 = V.1 ~ b.preis * alt1.x1,
    v2 = V.2 ~ b.preis * alt2.x1
  )
)


sedrive <- sim_all(
  nosim = nosim, resps = resps,
  designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups,
  manipulations = manipulations, utility_transform_type = "exact", mode = "sequential"
)

#' Aggregate Simulation Results
#'
#' Processes the simulation results to extract summaries, coefficients, and graphs.
#'
#' @param all_designs A list of simulation results from sim_choice. Can contain different designs but need to have the common structure returned by simchoice
#' @param fromfolder A folder from where to read simulations. If provided, the function will read all .qs files from the folder and process them. The files are usually saved by your earlier work and should be qs files as they are more efficient that rds files.
#' @return A list with aggregated results including summary, coefficients, graphs, and power.
#' @export
aggregateResults <- function(all_designs, fromfolder = NULL) {
  if (!is.null(fromfolder)) {
    if (!dir.exists(fromfolder)) stop("Folder from where to read simulations does not exist.")

    designs <- list.files(path = fromfolder, pattern = "*.qs", full.names = TRUE)

    purrr::map(designs, qs2::qs_read) %>% stats::setNames(basename(designs))
  }


  designname <- all_designs[["arguements"]][["designname"]]
  reshape_type <- all_designs[["arguements"]][["Reshape Type"]]
  bcoeff <- all_designs[["arguements"]][["Beta values"]]

  powa <- all_designs |>
    purrr::map(~ .x$power) |>
    purrr::compact()

  summaryall <- as.data.frame(purrr::compact(purrr::map(all_designs, ~ .x$summary))) %>% ## purrr::compact to remove all NULL
    dplyr::select(!dplyr::ends_with("vars")) %>%
    tibble::rownames_to_column("parname") %>%
    dplyr::mutate(parname = stringr::str_remove(parname, "^est_")) %>%
    dplyr::left_join(
      data.frame(truepar = unlist(bcoeff)) %>% tibble::rownames_to_column("parname") %>%
        dplyr::mutate(parname = stringr::str_replace_all(parname, "\\.", "_")), ## because parameters have been renamed for mixl, we have to make sure we substitute all . with _
      by = "parname"
    ) %>%
    dplyr::relocate(parname, dplyr::ends_with(c(
      ".n", "truepar", "mean", "sd", "min", "max", "range", "se"
    )))

  coefall <- purrr::compact(purrr::map(all_designs, ~ .x$coefs))

  pat <- paste0("(", paste(designname, collapse = "|"), ").") # needed to identify pattern to be replaced

  preprocessed <- as.data.frame(coefall) %>%
    dplyr::select(!dplyr::matches("pval|run")) %>%
    dplyr::rename_with(~ sub("est_", "", .x), dplyr::everything()) %>%
    dplyr::rename_with(
      ~ paste0(., "_", stringr::str_extract(., pat)),
      dplyr::everything()
    ) %>% # rename attributes for reshape part 1
    dplyr::rename_with(
      ~ stringr::str_replace(.,
        pattern = pat, replacement =
          ""
      ),
      dplyr::everything()
    )


  s <- switch(reshape_type,
    "stats" = {
      message("Using stats::reshape for reshaping...")
      stats::reshape(
        preprocessed,
        varying = 1:ncol(preprocessed),
        sep = "_",
        direction = "long",
        timevar = "design"
      ) %>%
        dplyr::select(-id)
    },
    "tidyr" = {
      message("Using tidyr::pivot_longer for reshaping...")
      tidyr::pivot_longer(
        preprocessed,
        cols = dplyr::everything(),
        names_to = c(".value", "design"),
        names_sep = "_",
        values_drop_na = TRUE
      )
    },
    "auto" = {
      tryCatch(
        {
          message("Trying tidyr::pivot_longer for reshaping...")
          tidyr::pivot_longer(
            preprocessed,
            cols = dplyr::everything(),
            names_to = c(".value", "design"),
            names_sep = "_",
            values_drop_na = TRUE
          )
        },
        error = function(e) {
          message("tidyr::pivot_longer failed, falling back to stats::reshape...")
          stats::reshape(
            preprocessed,
            varying = 1:ncol(preprocessed),
            sep = "_",
            direction = "long",
            timevar = "design"
          ) %>%
            dplyr::select(-id)
        }
      )
    },
    stop("You need to specify either 'tidyr', 'stats', or 'auto' as the reshape_type")
  )


  p <- list()

  for (att in names(dplyr::select(s, -c("design")))) {
    p[[att]] <- plot_multi_histogram(s, att, "design")

    print(p[[att]])
  }

  all_designs[["summaryall"]] <- summaryall
  all_designs[["graphs"]] <- p
  all_designs[["powa"]] <- powa


  return(all_designs)
}

#' Simulate choices based on a data.frame with a design and respondents
#'
#' @param data a dataframe that includes a design repeated for the number of observations
#' @param utility a list with the utility functions, one utility function for each alternatives
#' @param setspp  Deprecated. Ignored and retained for backward compatibility.
#' @param bcoeff List of initial coefficients for the utility function. List content/length can vary based on application. I ideally begins (but does not have to) with b and need be the same as those entered in the utility functions
#' @param decisiongroups A vector showing how decision groups are numerically distributed
#' @param manipulations A variable to alter terms of the utility functions examples may be applying a factor or applying changes to terms selectively for different groups
#' @param preprocess_function = NULL You can supply a function that reads in external data (e.g. GIS coordinates) that will be merged with the simulated dataset. Make sure the the function outputs a data.frame that has a variable called ID which is used for matching.
#' @return a data.frame that includes simulated choices and a design
#' @export
#' @import data.table
#' @examples
#' example_df <- data.frame(
#'   ID = rep(1:100, each = 4),
#'   price = rep(c(10, 10, 20, 20), 100),
#'   quality = rep(c(1, 2, 1, 2), 100)
#' )
#'
#' beta <- list(
#'   bprice   = -0.2,
#'   bquality =  0.8
#' )
#'
#' ut <- list(
#'   u1 = list(
#'     v1 = V.1 ~ bprice * price + bquality * quality,
#'     v2 = V.2 ~ 0
#'   )
#' )
#' simulate_choices(example_df, ut, setspp = 4, bcoeff = beta)
#'
simulate_choices <- function(data, utility, setspp, bcoeff, decisiongroups = c(0, 1), manipulations = list(),  preprocess_function = NULL) { # the part in dataset that needs to be repeated in each run



  if (!missing(setspp)) {
    warning(
      "`setspp` is deprecated and ignored. ",
      "The number of choice sets per respondent is inferred from `data`.",
      call. = FALSE
    )
  }

  if (!is.null(preprocess_function)) {
    if (!is.function(preprocess_function)) {
      stop("`preprocess_function` must be a function.")
    } else {
      # Execute the user-supplied `preprocess_function`
      prepro_data <- preprocess_function()
      if (!is.null(prepro_data) && (!is.data.frame(prepro_data) || !"ID" %in% names(prepro_data))) {
        stop("The output of `preprocess_function` must be a data.frame with a column named 'ID'.")
      }
      message("\n Preprocess function has been executed.\n")
    }
  } else {
    message("\n No preprocess function provided. Proceeding without additional preprocessing.\n")
  }


  # Check 1: user specified multiple decision groups
  if (length(decisiongroups) > 2 && length(utility) != (length(decisiongroups) - 1)) {
    stop(glue::glue(
      "Length of `utility` ({length(utility)}) does not match number of decision groups ",
      "defined by `decisiongroups` ({length(decisiongroups) - 1})."
    ))
  }

  tictoc::tic("whole simulate choices")

  tictoc::tic("assign keys for bcoeff")
  ### unpack the bcoeff list so variables are accessible
  for (key in names(bcoeff)) {
    assign(key, bcoeff[[key]])
  }

  message( utils::capture.output(tictoc::toc(log = FALSE, quiet = TRUE)) )

### new functions to calculate utility
  compile_one <- function(fm) {
    name <- as.character(formula.tools::lhs(fm))
    rhs  <- formula.tools::rhs(fm)
    fn   <- eval(bquote(function(d) with(d, .(rhs))))
    list(name = name, fun = compiler::cmpfun(fn))
  }

  compile_utility_list <- function(u) {
    lapply(u, function(fl) {
      tmp <- lapply(fl, compile_one)
      stats::setNames(lapply(tmp, `[[`, "fun"),
               vapply(tmp, `[[`, "", "name"))
    })
  }




  if (!exists("manipulations")) manipulations <- list() ## If no user input on further data manipulations

  n <- seq_along(1:length(utility[[1]])) # number of utility functions


  message("\n dataset preprossed_data exists: ", exists("prepro_data"), "\n")

  if (exists("prepro_data")) data <- dplyr::left_join(data, prepro_data, by = "ID")

  message("\n decisiongroups exists: ", length(decisiongroups) > 2)

  if (length(decisiongroups) > 2) { ### create a new variable to classify decision groups.

    data <- dplyr::mutate(data, group = as.numeric(cut(dplyr::row_number(),
      breaks = decisiongroups * dplyr::n(),
      labels = seq_along(decisiongroups[-length(decisiongroups)]),
      include.lowest = TRUE
    )))

    message(
      "\nGroup counts:\n",
      paste(utils::capture.output(print(table(data$group))), collapse = "\n")
    )
  } else {
    data$group <- 1
  }

  ### give an error if things did not work out

  if (any(!unique(data$group) %in% seq_along(utility))) {
    stop("Mismatch: data includes group IDs not covered by the utility list.")
  }

  tictoc::tic("user entered manipulations")

  ## Do user entered manipulations to choice set
  data <- data %>%
    dplyr::group_by(ID) %>%
    dplyr::mutate(!!!manipulations)
  message( utils::capture.output(tictoc::toc(log = FALSE, quiet = TRUE)) )


  tictoc::tic("for each group calculate utility")


  ufuns <- compile_utility_list(utility)

  dt <- data.table::as.data.table(data)



  all_vars <- unique(unlist(lapply(utility, function(fl) {
    unlist(lapply(fl, function(fm) all.vars(formula.tools::rhs(fm))))
  })))
  sdcols <- intersect(all_vars, names(dt))


  varn <- names(ufuns[[1]])

  dt[
    ,
    (varn) := lapply(ufuns[[.GRP]], function(f) f(.SD)),
    by = group,
    .SDcols = sdcols
  ]





  tictoc::tic("add random component")
  ## add random component and calculate total utility
  data <- dt %>%
    dplyr::ungroup() %>%
    dplyr::rename_with(~ stringr::str_replace_all(., pattern = "\\.", "_"), tidyr::everything()) %>%
    dplyr::mutate(
      dplyr::across(.cols = dplyr::all_of(n), .fns = ~ evd::rgumbel(dplyr::n(), loc = 0, scale = 1), .names = "{'e'}_{n}"),
      dplyr::across(dplyr::starts_with("V_"), .names = "{'U'}_{n}") + dplyr::across(dplyr::starts_with("e_"))
    )  %>%
    as.data.frame() %>%
    dplyr::mutate(CHOICE = max.col(.[, grep("U_", names(.))]))


  message( utils::capture.output(tictoc::toc(log = FALSE, quiet = TRUE)) )

  message( utils::capture.output(tictoc::toc(log = FALSE, quiet = TRUE)) )

  message("\n data has been created \n")

  message(
    "\nFirst few observations of the dataset\n",
    paste(utils::capture.output(utils::head(data)), collapse = "\n"),
    "\n\n"
  )

  return(data)
}

rm(list = ls())
devtools::load_all()

library(dplyr)
designpath <- system.file("extdata", "spdesigns", "designs", package = "simulateDCE")

destype <- "spdesign"
resps <- 4000 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

# betacoefficients should not include "-"
bcoeff <- list(
  bx1A = 0.1, ## very high asc
  bx1B = -0.1,
  bx2A = 0.2,
  bx2B = -0.2
)


desisiongroups <- c(0, 0.3, 1)

ul <- list(
  uA =
    list(
      v1 = V.1 ~ bx1A * alt1.x1 + bx2A * alt1.x2,
      v2 = V.2 ~ bx1A * alt2.x1 + bx2A * alt2.x2
    ),
  uB = list(
    v1 = V.1 ~ bx1B * alt1.x1 + bx2B * alt1.x2,
    v2 = V.2 ~ bx1B * alt2.x1 + bx2B * alt2.x2
  )
)

simplesim <- sim_all(nosim = nosim, resps = resps, designpath = designpath, bcoeff = bcoeff, u = ul, designtype = "spdesign", decisiongroups = desisiongroups)

testdata <- simplesim[["twoattr"]][[1]][["data"]] %>%
  distinct(group, Choice_situation, .keep_all = T) %>%
  arrange(Choice_situation)

## estimate models

library(apollo)

apollo_initialise()


apollo_control <- list(
  modelName       = "Clogit_simpledesign",
  modelDescr      = "Simple conditional logit model for group 1 ",
  indivID         = "ID",
  nCores          = 1,
  outputDirectory = "/~"
)


## group 1

database <- simplesim[["twoattr"]][[1]][["data"]]
apollo_beta <- c(
  b_x1 = 0,
  b_x2 = 0
)

apollo_fixed <- c()





# ################################################################# #
#### GROUP AND VALIDATE INPUTS                                   ####
# ################################################################# #

apollo_inputs <- apollo_validateInputs()

# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #

apollo_probabilities <- function(apollo_beta, apollo_inputs, functionality = "estimate") {
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))

  ### Create list of probabilities P
  P <- list()

  ### Define settings for MNL model component that are generic across classes
  mnl_settings <- list(
    alternatives = c(alt1 = 1, alt2 = 2),
    avail        = list(alt1 = 1, alt2 = 1),
    choiceVar    = CHOICE
  )




  ### Compute class-specific utilities
  V <- list()
  V[["alt1"]] <- b_x1 * alt1_x1 + b_x2 * alt1_x2
  V[["alt2"]] <- b_x1 * alt2_x1 + b_x2 * alt2_x2

  mnl_settings$utilities <- V
  # mnl_settings$componentName = paste0("Class_",s)



  ### Compute probabilities using MNL model
  P[["model"]] <- apollo_mnl(mnl_settings, functionality)

  ### Take product across observation for same individual
  P <- apollo_panelProd(P, apollo_inputs, functionality)

  ### Prepare and return outputs of function
  P <- apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# ################################################################# #
#### MODEL ESTIMATION                                            ####
# ################################################################# #

### Optional starting values search
# apollo_beta=apollo_searchStart(apollo_beta, apollo_fixed,apollo_probabilities, apollo_inputs)

model <- apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs, estimate_settings = list(estimationRoutine = "BFGS", hessianRoutine = "numDeriv"))



apollo_modelOutput(model)




### latent class model

apollo_control <- list(
  modelName       = "Latent Class logit_simpledesign",
  modelDescr      = "Latent Class  logit model",
  indivID         = "ID",
  nCores          = 1,
  outputDirectory = "/~"
)


## group 1

database <- simplesim[["twoattr"]][[1]][["data"]]

apollo_beta <- c(
  b_x1_a = 0,
  b_x2_a = 0,
  b_x1_b = 0,
  b_x2_b = 0,
  delta_a = 0,
  delta_b = 0
)

### Vector with names (in quotes) of parameters to be kept fixed at their starting value in apollo_beta, use apollo_beta_fixed = c() if none
apollo_fixed <- c("delta_b")

# ################################################################# #
#### DEFINE LATENT CLASS COMPONENTS                              ####
# ################################################################# #

apollo_lcPars <- function(apollo_beta, apollo_inputs) {
  lcpars <- list()

  lcpars[["b_x1"]] <- list(b_x1_a, b_x1_b)
  lcpars[["b_x2"]] <- list(b_x2_a, b_x2_b)

  V <- list()
  V[["class_a"]] <- delta_a
  V[["class_b"]] <- delta_b


  classAlloc_settings <- list(
    classes      = c(class_a = 1, class_b = 2),
    utilities    = V
  )

  lcpars[["pi_values"]] <- apollo_classAlloc(classAlloc_settings)

  return(lcpars)
}



# ################################################################# #
#### GROUP AND VALIDATE INPUTS                                   ####
# ################################################################# #

apollo_inputs <- apollo_validateInputs()

# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #


# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #

apollo_probabilities <- function(apollo_beta, apollo_inputs, functionality = "estimate") {
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))

  ### Create list of probabilities P
  P <- list()

  ### Define settings for MNL model component that are generic across classes
  mnl_settings <- list(
    alternatives = c(alt1 = 1, alt2 = 2),
    avail        = list(alt1 = 1, alt2 = 1),
    choiceVar    = CHOICE
  )

  ### Loop over classes
  for (s in 1:2) {
    ### Compute class-specific utilities
    V <- list()
    V <- list()
    V[["alt1"]] <- b_x1[[s]] * alt1_x1 + b_x2[[s]] * alt1_x2
    V[["alt2"]] <- b_x1[[s]] * alt2_x1 + b_x2[[s]] * alt2_x2



    mnl_settings$utilities <- V
    # mnl_settings$componentName = paste0("Class_",s)

    ### Compute within-class choice probabilities using MNL model
    P[[paste0("Class_", s)]] <- apollo_mnl(mnl_settings, functionality)

    ### Take product across observation for same individual
    P[[paste0("Class_", s)]] <- apollo_panelProd(P[[paste0("Class_", s)]], apollo_inputs, functionality)
  }

  ### Compute latent class model probabilities
  lc_settings <- list(inClassProb = P, classProb = pi_values)
  P[["model"]] <- apollo_lc(lc_settings, apollo_inputs, functionality)

  ### Prepare and return outputs of function
  P <- apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# ################################################################# #
#### MODEL ESTIMATION                                            ####
# ################################################################# #

### Optional starting values search
apollo_beta <- apollo_searchStart(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)

model <- apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs) # ,estimate_settings =list(estimationRoutine="bhhh" , hessianRoutine= "maxLik")

# ################################################################# #
#### MODEL OUTPUTS                                               ####
# ################################################################# #

# ----------------------------------------------------------------- #
#---- FORMATTED OUTPUT (TO SCREEN)                               ----
# ----------------------------------------------------------------- #

apollo_modelOutput(model)

rm(list = ls())
devtools::load_all()
### currently used to test the chunks option

set.seed(3393)

designpath <- system.file("extdata", "Rbook", package = "simulateDCE")
# notes <- "This design consists of different heuristics. One group did not attend the methan attribute, another group only decided based on the payment"

notes <- "No Heuristics"

resps <- 240 # number of respondents
nosim <- 19 # number of simulations to run (about 500 is minimum)

# betacoefficients should not include "-"
bcoeff <- list(
  bsq = 0.00,
  bredkite = -0.05,
  bdistance = 0.50,
  bcost = -0.05,
  bfarm2 = 0.25,
  bfarm3 = 0.50,
  bheight2 = 0.25,
  bheight3 = 0.50
)


destype <- "spdesign"


# place your utility functions here
ul <- list(u1 = list(
  v1 = V.1 ~ bsq * alt1.sq,
  v2 = V.2 ~ bfarm2 * alt2.farm2 + bfarm3 * alt2.farm3 + bheight2 * alt2.height2 + bheight3 * alt2.height3 + bredkite * alt2.redkite + bdistance * alt2.distance + bcost * alt2.cost,
  v3 = V.3 ~ bfarm2 * alt3.farm2 + bfarm3 * alt3.farm3 + bheight2 * alt3.height2 + bheight3 * alt3.height3 + bredkite * alt3.redkite + bdistance * alt3.distance + bcost * alt3.cost
))



rbook <- simulateDCE::sim_all(
  nosim = nosim, resps = resps, destype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, chunks = 4, utility_transform_type = "exact"
)

plot_multi_histogram <- function(df, feature, label_column, hist = FALSE) { # function to create nice multi histograms, taken somewhere from the web
  plt <- ggplot2::ggplot(df, ggplot2::aes(x = eval(parse(text = feature)), fill = eval(parse(text = label_column)))) +
    ggplot2::geom_density(alpha = 0.5) +
    ggplot2::geom_vline(ggplot2::aes(xintercept = mean(eval(parse(text = feature)))), color = "black", linetype = "dashed", linewidth = 1) + ## this makes a vertical line of the mean
    ggplot2::labs(x = feature, y = "Density") +
    ggplot2::guides(fill = ggplot2::guide_legend(title = label_column))

  if (hist == TRUE) plt + ggplot2::geom_histogram(alpha = 0.7, position = "identity", ggplot2::aes(y = ..density..), color = "black")

  return(plt)
}


## The function below is intended to find a dataframe called $coef in the output
## regardless of the output data structure, which can vary
find_dataframe <- function(list_object, dataframe_name) {
  # Check if the current object is a list
  if (is.list(list_object)) {
    # Check if the dataframe_name exists in the names of the current list object
    if (dataframe_name %in% names(list_object)) {
      # Check if the object corresponding to dataframe_name is a data frame
      if (is.data.frame(list_object[[dataframe_name]])) {
        return(list_object[[dataframe_name]]) # Return the data frame if found
      }
    }

    # Recursively search through each element of the current list object
    for (element in list_object) {
      result <- find_dataframe(element, dataframe_name)
      if (!is.null(result)) {
        return(result) # Return the data frame if found in any nested list
      }
    }
  }

  return(NULL) # Return NULL if dataframe_name not found
}



make_md <- function(f = file) {
  rmarkdown::render("simulation_output.rmd",
    output_file = paste0(
      stringr::str_remove_all(
        file, "parameters_|.R$"
      ), ".html"
    ),
    params = list(file = file)
  )
}



#' Modify bcoeff Names
#'
#' This function modifies the names of beta coefficients (`bcoeff`) by removing
#' dots and underscores to ensure consistent naming. If the names are already modified,
#' the function skips processing.
#'
#' @param bcoeff A named list of beta coefficients. The names of this list are
#'   the coefficients used in the utility function.
#'
#' @return A list containing:
#' \describe{
#'   \item{bcoeff}{The modified `bcoeff` list with updated names.}
#'   \item{bcoeff_lookup}{A tibble mapping the original names to the modified names.}
#' }
#'
#' @keywords internal
modify_bcoeff_names <- function(bcoeff) {
  # Check if bcoeff already has a lookup table attribute
  if (!is.null(attr(bcoeff, "bcoeff_lookup"))) {
    message("bcoeff_lookup already exists; skipping modification.")
    # Retrieve the existing lookup table
    bcoeff_lookup <- attr(bcoeff, "bcoeff_lookup")
  } else {
    # Check if bcoeff names need modification
    if (any(grepl("[._]", names(bcoeff)))) {
      # Create a lookup table
      bcoeff_lookup <- tibble::tibble(
        original = names(bcoeff),
        modified = stringr::str_replace_all(names(bcoeff), "[._]", "")
      )
      # Modify the names in the original bcoeff
      names(bcoeff) <- bcoeff_lookup$modified
    } else {
      # No modification needed; create a trivial lookup table
      bcoeff_lookup <- tibble::tibble(
        original = names(bcoeff),
        modified = names(bcoeff)
      )
    }
    # Attach the lookup table as an attribute to bcoeff
    attr(bcoeff, "bcoeff_lookup") <- bcoeff_lookup
  }

  # Return both modified bcoeff and lookup table
  list(
    bcoeff = bcoeff,
    bcoeff_lookup = bcoeff_lookup
  )
}

rm(list = ls())
devtools::load_all()


design <- system.file("extdata", "spdesigns", "designs", "twoattr.RDS", package = "simulateDCE")


resps <- 4000 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

# betacoefficients should not include "-"
bcoeff <- list(
  bx1A = 0.1, ## very high asc
  bx1B = -0.1,
  bx2A = 0.2,
  bx2B = -0.2
)


desisiongroups <- c(0, 0.3, 1)

ul <- list(
  uA =
    list(
      v1 = V.1 ~ bx1A * alt1.x1 + bx2A * alt1.x2,
      v2 = V.2 ~ bx1A * alt2.x1 + bx2A * alt2.x2
    ),
  uB = list(
    v1 = V.1 ~ bx1B * alt1.x1 + bx2B * alt1.x2,
    v2 = V.2 ~ bx1B * alt2.x1 + bx2B * alt2.x2
  )
)

simplesim <- sim_all(nosim = nosim, resps = resps, designpath = designpath, bcoeff = bcoeff, u = ul, designtype = "spdesign", decisiongroups = desisiongroups)

formattedes <- readdesign(design = design)

simulate_choices()

#' Title Extracts beta values from an spdesign object
#'
#' @param input_list the list where the parameters are stored. Usually this is `design$utility`
#'
#' @return A named list with parameter values which can be used in `sim_all`
#' @export
#'
#' @examples
#' d <- system.file("extdata", "CSA", "linear", "BLIeff.RDS", package = "simulateDCE")
#' extract_b_values(readRDS(d)$utility)
#'
extract_b_values <- function(input_list) {
  extract_b_value <- function(input_string) {
    matches <- gregexpr("b_\\w+\\[(-?\\d+\\.?\\d*)\\]", input_string)
    matches <- regmatches(input_string, matches)
    matches <- unlist(matches)


    names <- gsub("\\[.*", "", matches)

    values <- gsub(".*\\[(-?\\d+\\.?\\d*)\\]", "\\1", matches)

    result <- stats::setNames(as.numeric(values), names)
    return(result)
  }

  b_values <- unlist(lapply(input_list, extract_b_value))
  b_values <- stats::setNames(b_values, gsub("alt\\d+\\.", "", names(b_values)))
  return(as.list(b_values))
}

#'  Creates a dataframe with the design.
#'
#' @param design The path to a design file
#' @param designtype Is it a design created with ngene, spdesign or idefix. use 'ngene', 'spdesign' or 'idefix. Ngene designs should be stored as the standard .ngd output. spdesign should be the spdesign object stored as an RDS file. Idefix objects should also be stored as an RDS file. If designtype is not specified, I try to guess what it is. This is especially helpful if you want to carry out a simulation for both spdesign designs and ngene designs at the same time.
#' @param destype Deprecated. Use designtype instead.
#' @return a dataframe
#' @export
#'
#' @examples library(simulateDCE)
#' mydesign <- readdesign(
#'   system.file("extdata", "agora", "altscf_eff.ngd", package = "simulateDCE"),
#'   "ngene"
#' )
#'
#' print(mydesign)
#'
readdesign <- function(design = designfile, designtype = NULL, destype = NULL) {
  # Ensure both designtype and destype are not set simultaneously
  if (!is.null(designtype) && !is.null(destype)) {
    stop("Both 'designtype' and 'destype' cannot be specified at the same time. Please use only 'designtype'.")
  }

  # Handle deprecated 'destype' argument
  if (!is.null(destype)) {
    message("'destype' is deprecated. Please use 'designtype' instead.")
    designtype <- destype
  }



  if (is.null(designtype)) {
    # Check if the string ends with ".ngd"
    if (grepl("\\.ngd$", design)) {
      # Code to execute if condition is true
      designtype <- "ngene"
      message("I guessed it is an ngene file")
    } else if ("spdesign" %in% class(readRDS(design))) {
      designtype <- "spdesign"
      message("I assume it is a spdesign.")
    } else {
      designtype <- "idefix"
      message("I assume it is an idefix design.")
    }
  }


  read_test <- function() {
    designf <- readRDS(design)
    if (is.list(designf) & !is.data.frame(designf)) {
      if (!"design" %in% names(designf)) {
        stop("The 'design' list element is missing. Make sure to provide a proper spdesign object.")
      }
      designf <- designf[["design"]] %>%
        as.data.frame()
    }
    as.data.frame(designf)
  }
  idefix <- function() {
    # Process the data
    read_test() %>%
      tibble::rownames_to_column(var = "row_id") %>%
      dplyr::filter(!grepl("no.choice", row_id)) %>% # Exclude no.choice rows
      dplyr::select(!dplyr::contains("cte")) %>% # Drop unnecessary columns
      dplyr::mutate(
        # Extract Choice.situation as number after 'set'
        Choice.situation = as.integer(sub("^set(\\d+).*", "\\1", row_id)),
        # Extract alt as the alternative identifier
        alt = sub(".*\\.", "", row_id)
      ) %>%
      dplyr::select(-"row_id") %>% # Drop the original row_id
      tidyr::pivot_wider(
        id_cols = "Choice.situation", # Group by Choice.situation
        names_from = "alt", # Use alt to create column suffixes
        values_from = -c("Choice.situation", "alt"), # Values from other columns
        names_glue = "{alt}.{.value}" # Custom naming convention
      )
  }

  design <- switch(designtype,
    "ngene" = suppressWarnings(readr::read_delim(design,
      delim = "\t",
      escape_double = FALSE,
      trim_ws = TRUE,
      col_select = c(-Design, -tidyr::starts_with("...")),
      name_repair = "universal", show_col_types = FALSE, guess_max = Inf
    )) %>%
      dplyr::filter(!is.na(Choice.situation)),
    "spdesign" = {
      read_test() %>%
        dplyr::mutate(Choice.situation = 1:dplyr::n()) %>%
        dplyr::rename_with(~ stringr::str_replace(., pattern = "_", "\\."), tidyr::everything()) %>%
        dplyr::rename_with(~ dplyr::case_when(
          . == "block" ~ "Block",
          TRUE ~ .
        ), tidyr::everything())
    },
    "idefix" = idefix(),
    stop("Invalid value for design. Please provide either NULL, 'ngene', 'spdesign'or 'idefix',  or do not use the argument 'designtype'. NULL lets us to guess the design.")
  )
}

design_path <- system.file("extdata", "agora", "altscf_eff.ngd", package = "simulateDCE")


test_that("wrong designtype", {
  expect_error(readdesign(design = design_path, designtype = "ng"), "Invalid value for design. Please provide either NULL, 'ngene', 'spdesign'or 'idefix',  or do not use the argument 'designtype'. NULL lets us to guess the design.")
})


test_that("file does not exist", {
  expect_error(
    readdesign(design = system.file("data-raw/agora/alcf_eff.ngd", package = "simulateDCE"), designtype = "ngene"),
    "does not exist in current working directory"
  )
})

test_that("all is correct", {
  expect_no_error(readdesign(design = design_path, designtype = "ngene"))
})

# test if autodetect ngd design

test_that("expect message of guess", {
  expect_message(readdesign(design = design_path), "I guessed it is an ngene file")
})


test_that("with or without autodetct get same results for ngene", {
  t <- readdesign(design_path)
  t2 <- readdesign(design_path, designtype = "ngene")

  expect_equal(t, t2)
})


### Tests for spdesign

design_path <- system.file("extdata", "CSA", "linear", "BLIbay.RDS", package = "simulateDCE")

test_that("all is correct", {
  expect_no_error(readdesign(design = design_path, designtype = "spdesign"))
})






# Same Tests for spdesign, but detect automatically if it is spdesign

test_that("prints message for guessing", {
  expect_message(readdesign(design = design_path), "I assume it is a spdesign")
})

test_that("with or without autodetct get same results for spdesign", {
  t <- readdesign(design_path)
  t2 <- readdesign(design_path, designtype = "spdesign")

  expect_equal(t, t2)
})


## trying objects that do not work

design_path <- system.file("extdata", "testfiles", "nousefullist.RDS", package = "simulateDCE")

test_that("spdesign object is a list but does not contain the right element design", {
  expect_error(
    readdesign(design = design_path, designtype = "spdesign"),
    "list element is missing. Make sure to provide a "
  )
})


## test spdesign object containing original object

design_path <- system.file("extdata", "ValuGaps", "des1.RDS", package = "simulateDCE")

test_that("all is correct with full spdesign objects", {
  expect_no_error(readdesign(design = design_path, designtype = "spdesign"))
})



### Tests for idefix

design_idefix <- system.file("extdata", "Idefix_designs", "test_design2.RDS", package = "simulateDCE")



test_that("all is correct with full idefix objects", {
  expect_no_error(readdesign(design_idefix, designtype = "idefix"))
})

#### new tests with kind help from chatgpt


library(testthat)
library(simulateDCE)

test_that("readdesign correctly identifies and processes ngene files", {
  # Arrange
  design_ngene <- system.file("extdata", "agora", "altscf_eff.ngd", package = "simulateDCE")

  # Act
  result <- readdesign(design_ngene)

  # Assert
  expect_s3_class(result, "data.frame") # Check the result is a data frame
  expect_true("Choice.situation" %in% colnames(result)) # Verify key column presence

  # Ensure there are alternative-related columns
  alt_columns <- colnames(result)[grepl("^alt", colnames(result))]
  expect_gt(length(alt_columns), 0) # Confirm at least one alt-related column exists

  # Validate the structure of the output
  expect_gt(nrow(result), 0) # Ensure the data frame has rows
  expect_gt(ncol(result), 1) # Ensure the data frame has more than one column
  expect_message(readdesign(design_ngene), "I guessed it is an ngene file")
})

test_that("readdesign correctly identifies and processes spdesign files", {
  # Arrange
  design_sp <- system.file("extdata", "ValuGaps", "des1.RDS", package = "simulateDCE")

  # Act
  result_default <- readdesign(design_sp)
  result_explicit <- readdesign(design_sp, designtype = "spdesign")

  # Assert
  expect_s3_class(result_default, "data.frame") # Check the result is a data frame
  expect_s3_class(result_explicit, "data.frame")
  expect_identical(result_default, result_explicit) # Default and explicit designtype should match
  expect_true("Choice.situation" %in% colnames(result_default)) # Verify key column presence

  # Ensure there are alternative-related columns
  alt_columns <- colnames(result_default)[grepl("^alt", colnames(result_default))]
  expect_gt(length(alt_columns), 0) # Confirm at least one alt-related column exists

  # Validate the structure of the output
  expect_gt(nrow(result_default), 0) # Ensure the data frame has rows
  expect_gt(ncol(result_default), 1) # Ensure the data frame has more than one column
  expect_message(readdesign(design_sp), "I assume it is a spdesign.")
})

test_that("readdesign correctly identifies and processes idefix files", {
  # Arrange
  design_idefix <- system.file("extdata", "Idefix_designs", "test_design2.RDS", package = "simulateDCE")

  # Act
  result <- readdesign(design_idefix)

  # Assert
  expect_s3_class(result, "data.frame") # Check the result is a data frame
  expect_true("Choice.situation" %in% colnames(result)) # Verify key column presence

  # Ensure there are alternative-related columns
  alt_columns <- colnames(result)[grepl("^alt", colnames(result))]
  expect_gt(length(alt_columns), 0) # Confirm at least one alt-related column exists

  # Validate the structure of the output
  expect_gt(nrow(result), 0) # Ensure the data frame has rows
  expect_gt(ncol(result), 1) # Ensure the data frame has more than one column
  expect_message(readdesign(design_idefix), "I assume it is an idefix design.")
})



test_that("readdesign returns a data frame with proper structure", {
  # Arrange
  design_ngene <- system.file("extdata", "agora", "altscf_eff.ngd", package = "simulateDCE")

  # Act
  result <- readdesign(design_ngene)

  # Assert
  expect_s3_class(result, "data.frame") # Check the result is a data frame
  expect_true("Choice.situation" %in% colnames(result))

  # Ensure there are alternative-related columns
  alt_columns <- colnames(result)[grepl("^alt", colnames(result))]
  expect_gt(length(alt_columns), 0) # Confirm at least one alt-related column exists

  # Validate the structure of the output
  expect_gt(nrow(result), 0) # Ensure the data frame has rows
  expect_gt(ncol(result), 1) # Ensure the data frame has more than one column
})

rm(list = ls())
devtools::load_all()





set.seed(3393)

designpath <- system.file("extdata", "CSA", "linear", package = "simulateDCE")

library(rlang)
library(formula.tools)







notes <- "No Heuristics"

resps <- 240 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

destype <- "spdesign"

bcoeff <- list(
  bx1 = -0.02,
  bx2 = -0.03,
  bx3 = -0.02,
  bx4 = -0.005,
  bnobuy = -1
)

# place your utility functions here
ul <- list(u1 = list(
  v1 = V.1 ~ bx1 * alt1.x1 + bx2 * alt1.x2 + bx3 * alt1.x3 + bx4 * alt1.x4,
  v2 = V.2 ~ bx1 * alt2.x1 + bx2 * alt2.x2 + bx3 * alt2.x3 + bx4 * alt2.x4,
  v3 = V.3 ~ bnobuy
))

savefile <- "testdir/file"

csa <- simulateDCE::sim_all(
  nosim = nosim, resps = resps, designtype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, utility_transform_type = "exact", savefile =
    NULL, mode = "sequential"
)

rm(list = ls())
# devtools::load_all()


designpath <- system.file("extdata", "agora", package = "simulateDCE")

resps <- 360 # number of respondents
nosim <- 10 # number of simulations to run (about 500 is minimum)

# betacoefficients should not include "-"

# design priors parameters
bcoeff <- list(
  basc = -1.2,
  basc2 = -1.4,
  baction = 0.1,
  badvisory = 0.4,
  bpartnertest = 0.3,
  bcomp = 0.02
)


# from survey

# basc = -2.2
# basc2 = -2.7
# baction = -0.15
# badvisory = 0.26
# bpartner = -0.09
# bcomp = 0.004



# place your utility functions here
ul <- list(
  u1 =
    list(
      v1 = V.1 ~ basc + baction * alt1.b + badvisory * alt1.c + bpartnertest * alt1.d + bcomp * alt1.p, # Utility of alternative 1
      v2 = V.2 ~ basc2 + baction * alt2.b + badvisory * alt2.c + bpartnertest * alt2.d + bcomp * alt2.p, # Utility of alternative 2
      v3 = V.3 ~ 0
    )
)

destype <- "ngene"

tictoc::tic("Total length Agora")

agora <- simulateDCE::sim_all(
  nosim = nosim, resps = resps, designtype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, utility_transform_type = "exact", mode = "parallel"
)

tictoc::toc()

rm(list = ls())
devtools::load_all()



designpath <- system.file("extdata", "SE_AGRI", package = "simulateDCE")

destype <- "ngene"
resps <- 360 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

# betacoefficients should not include "-"
bcoeff <- list(
  basc = 4.2, ## very high asc
  bprof = 0.3,
  bexp = 0.3,
  bdomestic = 0.3,
  bforeign = 0.3,
  bdamage = 0.6,
  bprice = 0.2
)



manipulations <- list(
  alt1.professional = rlang::expr(alt1.initiator == 1),
  alt2.professional = rlang::expr(alt2.initiator == 1),
  alt1.expert = rlang::expr(alt1.initiator == 2),
  alt2.expert = rlang::expr(alt2.initiator == 2),
  alt1.domestic = rlang::expr(alt1.funding == 1),
  alt2.domestic = rlang::expr(alt2.funding == 1),
  alt1.foreign = rlang::expr(alt1.funding == 2),
  alt2.foreign = rlang::expr(alt2.funding == 2)
)


# place your utility functions here
ul <- list(
  u1 =
    list(
      v1 = V.1 ~ bprof * alt1.professional + bexp * alt1.expert + bdomestic * alt1.domestic + bforeign * alt1.foreign + bdamage * alt1.damage + bprice * alt1.compensation,
      v2 = V.2 ~ bprof * alt2.professional + bexp * alt2.expert + bdomestic * alt2.domestic + bforeign * alt2.foreign + bdamage * alt2.damage + bprice * alt2.compensation,
      v3 = V.3 ~ basc
    )
)

seagri <- sim_all(
  nosim = nosim, resps = resps, designtype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, manipulations = manipulations, utility_transform_type = "exact"
)

# ─────────────────────────────────────────────────────────────────────────────
# BENCHMARK in pure data.table, no back-and-forth to data.frame
# ─────────────────────────────────────────────────────────────────────────────

library(formula.tools)  # lhs() / rhs()
library(compiler)        # cmpfun()
library(purrr)           # map(), map2()
library(data.table)      # as.data.table(), .GRP, rbindlist(), setcolorder()
library(tictoc)          # tic()/toc()

# 1) Prepare data ------------------------------------------------------------
set.seed(2025)
n    <- 1e8
ngrp <- 10
df <- data.frame(
  price   = runif(n, 5, 100),
  quality = runif(n, 1,   5),
  group   = rep(1:ngrp, length.out = n)
)
bprice   <- -0.2
bquality <-  0.8

single_group <- list(
  V1 = V1 ~ bprice * price + bquality * quality,
  V2 = V2 ~ 0
)
utility_list <- rep(list(single_group), ngrp)

# 2) Helpers ---------------------------------------------------------------

compile_one <- function(fm) {
  nm  <- as.character(lhs(fm))
  rhs <- rhs(fm)
  fn  <- eval(bquote(function(d) with(d, .(rhs))))
  list(name = nm, fun = cmpfun(fn))
}

compile_utility_list <- function(ul) {
  lapply(ul, function(fl) {
    tmp <- lapply(fl, compile_one)
    setNames(lapply(tmp, `[[`, "fun"),
             vapply(tmp, `[[`, "", "name"))
  })
}

# 3) Four methods, all returning data.tables -------------------------------

old_group <- function(data, utility) {
  tic("OLD-GROUP")
  subs <- split(data, data$group)
  out  <- map2(utility, subs, ~ {
    dt <- as.data.table(.y)
    for (fm in .x) {
      dt[, as.character(lhs(fm)) := eval(rhs(fm), dt)]
    }
    dt
  })
  res <- rbindlist(out)
  toc(log = FALSE)
  res
}

new1_group <- function(data, utility) {
  ufuns <- compile_utility_list(utility)
  tic("NEW1-GROUP")
  subs  <- split(data, data$group)
  out   <- map2(ufuns, subs, ~ {
    dt <- as.data.table(.y)
    for (nm in names(.x)) dt[, (nm) := .x[[nm]](dt)]
    dt
  })
  res <- rbindlist(out)
  toc(log = FALSE)
  res
}

new2_group <- function(data, utility) {
  ufuns  <- compile_utility_list(utility)
  dt     <- as.data.table(data)
  groups <- sort(unique(dt$group))
  gm     <- setNames(seq_along(groups), groups)

  tic("NEW2-GROUP (loop)")
  for (g in groups) {
    idx <- gm[as.character(g)]
    fns <- ufuns[[idx]]
    dt[group == g, (names(fns)) := lapply(fns, function(f) f(.SD))]
  }
  toc(log = FALSE)
  dt
}

new3_group <- function(data, utility) {
  ufuns <- compile_utility_list(utility)
  dt    <- as.data.table(data)

  # figure which columns your utilities will actually use
  sdcols <- intersect(
    names(dt),
    unique(unlist(lapply(utility, function(fl) {
      unlist(lapply(fl, function(fm) all.vars(rhs(fm))))
    })))
  )
  varn <- names(ufuns[[1]])

  tic("NEW3-GROUP (single dt[ , by=group ])")
  out <- dt[
    ,
    c(.SD,
      setNames(lapply(ufuns[[.GRP]], function(f) f(.SD)), varn)
    ),
    by      = group,
    .SDcols = sdcols
  ]
  toc(log = FALSE)

  # reorder columns to exactly: (sdcols, group, new vars)
  setcolorder(out, c(sdcols, "group", varn))
  out
}

# 4) Run & compare ---------------------------------------------------------

res_old  <- old_group(df, utility_list)
res_new1 <- new1_group(df, utility_list)
res_new2 <- new2_group(df, utility_list)
res_new3 <- new3_group(df, utility_list)

# numeric‐accuracy check (ignores type/class)
stopifnot(
  isTRUE(all.equal(res_old,  res_new1)),
  isTRUE(all.equal(res_new1, res_new2)),
  isTRUE(all.equal(res_new2, res_new3))
)
message("✅ All four methods agree (within numeric tolerance).")

#–– 1) small toy dataset & simple utility for testing ––#
df_small <- data.frame(
  ID      = rep(1:5, each = 2),
  price   = rep(c(10, 20), 5),
  quality = rep(c(1,  2), 5)
)

beta <- list(
  bprice   = -1,
  bquality =  2
)

# note: use V.1 / V.2 so that rename_with("\\.", "_") => V_1/V_2
ut <- list(
  u1 = list(
    v1 = V.1 ~ bprice * price + bquality * quality,
    v2 = V.2 ~ 0
  )
)

#––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––#


simulate_choices(
    data = df_small,
    utility = ut,
    setspp = 2,
    bcoeff = beta
  )



### with decision groups

simulate_choices(
  data = df_small,
  utility = list(
    u1 = list(
      v1 = V.1 ~ bprice * price + bquality * quality,
      v2 = V.2 ~ 0
    ),
    u2 = list(
      v1 = V.1 ~ bprice * price + bquality * quality * 2,  # altered for group 2
      v2 = V.2 ~ 0
    )
  ),
  setspp = 2,
  bcoeff = beta,
  decisiongroups = c(0, 0.5, 1)
)

#' Is a wrapper for sim_choice executing the simulation over all designs stored in a specific folder
#' update
#' @param nosim Number of runs or simulations. For testing use 2 but once you go serious, use at least 200, for better results use 2000.
#' @param resps Number of respondents you want to simulate
#' @inheritParams readdesign
#' @param designpath The path to the folder where the designs are stored. For example "c:/myfancydec/Designs"
#' @param reshape_type Must be "auto", "stats" to use the reshape from the stats package or tidyr to use pivot longer. Default is auto and should not bother you. Only change it once you face an error at this position and you may be lucky that it works then.
#' @inheritParams sim_choice
#' @inheritParams simulate_choices
#' @return A list, with all information on the simulation. This list an be easily processed by the user and in the rmarkdown template.
#' @export
#'
#' @examples
#' library(rlang)
#' designpath <- system.file("extdata", "SE_DRIVE", package = "simulateDCE")
#' resps <- 120 # number of respondents
#' nosim <- 2 # number of simulations to run (about 500 is minimum)
#'
#' decisiongroups <- c(0, 0.7, 1)
#'
#' # pass beta coefficients as a list
#' bcoeff <- list(
#'   b.preis = -0.01,
#'   b.lade = -0.07,
#'   b.warte = 0.02
#' )
#'
#' manipulations <- list(
#'   alt1.x2 = expr(alt1.x2 / 10),
#'   alt1.x3 = expr(alt1.x3 / 10),
#'   alt2.x2 = expr(alt2.x2 / 10),
#'   alt2.x3 = expr(alt2.x3 / 10)
#' )
#'
#'
#' # place your utility functions here
#' ul <- list(
#'   u1 =
#'
#'     list(
#'       v1 = V.1 ~ b.preis * alt1.x1 + b.lade * alt1.x2 + b.warte * alt1.x3,
#'       v2 = V.2 ~ b.preis * alt2.x1 + b.lade * alt2.x2 + b.warte * alt2.x3
#'     ),
#'   u2 = list(
#'     v1 = V.1 ~ b.preis * alt1.x1,
#'     v2 = V.2 ~ b.preis * alt2.x1
#'   )
#' )
#'
#'
#' sedrive <- sim_all(
#'   nosim = nosim,
#'   resps = resps,
#'   designpath = designpath,
#'   u = ul,
#'   bcoeff = bcoeff,
#'   decisiongroups = decisiongroups,
#'   manipulations = manipulations,
#'   utility_transform_type = "exact",
#'   mode = "sequential",
#'   estimate=FALSE
#' )
#'
sim_all <- function(nosim = 2,
                    resps,
                    designtype = NULL,
                    destype = NULL,
                    designpath,
                    u,
                    bcoeff,
                    decisiongroups = c(0, 1),
                    manipulations = list(),
                    estimate = TRUE,
                    chunks = 1,
                    utility_transform_type = "simple",
                    reshape_type = "auto",
                    mode = c("parallel", "sequential"),
                    preprocess_function = NULL,
                    savefile = NULL) {
  #################################################
  ########## Input Validation Test ###############
  #################################################
  mode <- match.arg(mode)
  ########### validate the utility function ########
  if (missing(u) || !(is.list(u) && any(sapply(u, is.list)))) {
    stop(
      " 'u' must be provided and must be a list containing at least one list element (list of lists)."
    )
  }

  ########## validate the bcoeff list ################
  # Check if bcoeff is provided
  if (missing(bcoeff)) {
    stop("Argument 'bcoeff' is required.")
  }


  if (nosim < chunks) {
    stop(
      "You cannot have more chunks than runs. The number of chunks tells us how often we save the simulation results on disk. Maximum one per run."
    )
  }

  # Check if bcoeff is a list
  if (!is.list(bcoeff)) {
    stop("Argument 'bcoeff' must be a list.")
  }

  if (length(u) != length(decisiongroups) - 1) {
    stop("Number of decision groups must equal number of utility functions!")
  }
  if (!is.vector(decisiongroups)) {
    stop("Decision groups must be a vector.")
  }

  # Check if decisiongroups starts with 0
  if (decisiongroups[1] != 0) {
    stop("Decision groups must start with 0.")
  }

  # Check if decisiongroups ends with 1
  if (utils::tail(decisiongroups, 1) != 1) {
    stop("Decision groups must end with 1.")
  }


  # Check if values in bcoeff are numeric
  if (!all(sapply(bcoeff, is.numeric))) {
    stop("Values in 'bcoeff' must be numeric.")
  }

  #### check that all the coefficients in utility function have a corresponding value in bcoeff ####
  # Extract coefficients from utility function starting with "b"
  coeff_names_ul <- unique(unlist(lapply(u, function(u) {
    formula_strings <- unlist(u)
    coef_names <- unique(unlist(lapply(formula_strings, function(f) {
      # Parse the formula to extract coefficient names
      all_vars <- all.vars(stats::as.formula(f))
      coef_vars <- all_vars[grep("^b", all_vars)]
      return(coef_vars)
    })))
    return(coef_names)
  })))

  # Check if all utility function coefficients starting with "b" are covered in bcoeff list
  missing_coeffs <- coeff_names_ul[!(coeff_names_ul %in% names(bcoeff))]
  if (length(missing_coeffs) > 0) {
    stop(paste(
      "Missing coefficients in 'bcoeff':",
      paste(missing_coeffs, collapse = ", "),
      ". Perhaps there is a typo?"
    ))
  }
  ########## validate resps #####################
  if (missing(resps) ||
    !(is.integer(resps) ||
      (is.numeric(resps) && identical(trunc(resps), resps)))) {
    stop(
      " 'resps' must be provided and must be an integer indicating  the number of respondents per run."
    )
  }
  ########## validate designpath ################
  if (!dir.exists(designpath)) {
    stop(
      " The folder where your designs are stored does not exist. \n Check if designpath is correctly specified"
    )
  }

  #################################################
  ########## End Validation Tests #################
  #################################################


  bcoeff_result <- modify_bcoeff_names(bcoeff)
  bcoeff <- bcoeff_result$bcoeff



  designfile <- list.files(designpath, full.names = T)
  designname <- stringr::str_remove_all(list.files(designpath, full.names = F), "(.ngd|_|.RDS)") ## Make sure designnames to not contain file ending and "_", as the may cause issues when replace


  tictoc::tic("total time for simulation and estimation")

  if (is.null(savefile)) {
    all_designs <- purrr::map(
      designfile,
      sim_choice,
      no_sim = nosim,
      respondents = resps,
      designtype = designtype,
      destype = destype,
      u = u,
      bcoeff = bcoeff,
      decisiongroups = decisiongroups,
      manipulations = manipulations,
      estimate = estimate,
      chunks = chunks,
      utility_transform_type = utility_transform_type,
      mode = mode,
      preprocess_function = preprocess_function,
      savefile = NULL
    ) %>% ## iterate simulation over all designs
      stats::setNames(designname)
  } else {
    purrr::walk(
      designfile,
      sim_choice,
      no_sim = nosim,
      respondents = resps,
      designtype = designtype,
      destype = destype,
      u = u,
      bcoeff = bcoeff,
      decisiongroups = decisiongroups,
      manipulations = manipulations,
      estimate = estimate,
      chunks = chunks,
      utility_transform_type = utility_transform_type,
      mode = mode,
      preprocess_function = preprocess_function,
      savefile = savefile
    )
    gc()

    all_designs <- purrr::map(list.files(dirname(savefile), full.names = TRUE), qs2::qs_read) %>%
      stats::setNames(designname)
  }

  time <- tictoc::toc()

  message(paste(utils::capture.output(print(time)), collapse = "\n"))


  all_designs[["time"]] <- time
  all_designs[["arguements"]] <- list(
    "Beta values" = bcoeff,
    "Utility functions" = u,
    "Decision groups" = decisiongroups,
    "Manipulation of vars" = manipulations,
    "Number Simulations" = nosim,
    "Respondents" = resps,
    "Designpath" = designpath,
    "Reshape Type" = reshape_type,
    "mode" = mode,
    "designname" = designname
  )

  if (estimate == TRUE) {
    all_designs <- simulateDCE::aggregateResults(all_designs = all_designs)
  }





  return(all_designs)
}

library(formula.tools)

## all tests are wrapped in single function to make it easier to call on different designs,
## which is done at the end of this script

comprehensive_design_test <- function(nosim, resps, designtype, designpath, ul, bcoeff, decisiongroups = c(0, 1)) {
  # Test cases related to sim_all function
  test_that("u is not a list of lists", {
    expect_error(
      sim_all(
        nosim = nosim, resps = resps, designtype = destype,
        designpath = designpath, u = data.frame(u = " alp"), bcoeff = bcoeff
      ),
      "must be provided and must be a list containing at least one list"
    )
  })

  test_that("no value provided for  utility", {
    expect_error(
      sim_all(
        nosim = nosim, resps = resps, designtype = destype,
        designpath = designpath, bcoeff = bcoeff
      ),
      "must be provided and must be a list containing at least one list"
    )
  })

  test_that("wrong designtype", {
    expect_error(sim_all(
      nosim = nosim, resps = resps, designtype = "ng",
      designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups
    ), "Invalid value for design. Please provide either NULL, 'ngene', 'spdesign'or 'idefix',  or do not use the argument 'designtype'. NULL lets us to guess the design.")
  })


  test_that("folder does not exist", {
    expect_error(
      sim_all(
        nosim = nosim, resps = resps, designtype = destype,
        designpath = system.file("da/bullshit", package = "simulateDCE"), u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups
      ),
      "The folder where your designs are stored does not exist."
    )
  })

  test_that("seed setting makes code reproducible", {
    set.seed(3333)
    result1 <- sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups)

    set.seed(3333)
    result2 <- sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups)

    expect_identical(result1[["summaryall"]], result2[["summaryall"]])
  })

  test_that("No seed setting makes code results different", {
    result1 <- sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups)


    result2 <- sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups)

    expect_failure(expect_identical(result1[["summaryall"]], result2[["summaryall"]]))
  })

  test_that("exact and simple produce same results", {
    set.seed(3333)
    result1 <- sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups, utility_transform_type = "simple")

    set.seed(3333)
    result2 <- sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups, , utility_transform_type = "exact")

    expect_identical(result1[["summaryall"]], result2[["summaryall"]])
  })


  test_that("Length of utility functions matches number of decision groups", {
    # Define test inputs
    badbcoeff <- list(
      basc = 0.2,
      bcow = 0.3,
      badv = 0.3,
      bvet = 0.3,
      bfar = 0.3,
      bmet = 0.3,
      bbon = 0.3,
      bbon2 = 1.9,
      basc2 = 2
    )

    badlist <- list(
      u1 = list(
        v1 = V.1 ~ bcow * alt1.cow + badv * alt1.adv + bvet * alt1.vet + bfar * alt1.far + bmet * alt1.met + bbon * alt1.bon,
        v2 = V.2 ~ bcow * alt2.cow + badv * alt2.adv + bvet * alt2.vet + bfar * alt2.far + bmet * alt2.met + bbon * alt2.bon,
        v3 = V.3 ~ basc
      ),
      u2 = list(
        v1 = V.1 ~ bcow * alt1.cow + badv * alt1.adv + bvet * alt1.vet + bfar * alt1.far + bbon * alt1.bon,
        v2 = V.2 ~ bcow * alt2.cow + badv * alt2.adv + bvet * alt2.vet + bfar * alt2.far + bbon * alt2.bon,
        v3 = V.3 ~ basc
      ),
      u3 = list(
        v1 = V.1 ~ bbon2 * alt1.bon,
        v2 = V.2 ~ bbon2 * alt2.bon,
        v3 = V.3 ~ basc
      ),
      u4 = list(
        v1 = V.1 ~ basc2 + bcow * alt1.cow + badv * alt1.adv + bvet * alt1.vet + bfar * alt1.far + bmet * alt1.met + bbon * alt1.bon,
        v2 = V.2 ~ bcow * alt2.cow + badv * alt2.adv + bvet * alt2.vet + bfar * alt2.far + bmet * alt2.met + bbon * alt2.bon,
        v3 = V.3 ~ basc
      )
    )
    baddecisiongroups <- c(0, 0.3, 0.6, 1)

    # Test that the function throws an error when lengths don't match
    expect_error(sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, u = badlist, bcoeff = badbcoeff, decisiongroups = baddecisiongroups), "Number of decision groups must equal number of utility functions!")

    # Define test inputs where lengths match
    gooddecisiongroups <- c(0, 0.3, 0.6, 0.8, 1)

    # Test that the function does not throw an error when lengths match (assumed true in input)
    expect_no_error(sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups))
  })


  ########### Additional Tests ##############
  test_that("bcoeff is provided", {
    expect_error(sim_all(
      nosim = nosim, resps = resps, designtype = destype,
      designpath = designpath, u = ul
    ))
  })

  test_that("bcoeff contains valid values", {
    expect_error(sim_all(
      nosim = nosim, resps = resps, designtype = destype,
      designpath = designpath, u = ul, bcoeff = list(bsq = "invalid")
    ))
  })

  test_that("bcoeff is a list", {
    expect_error(sim_all(
      nosim = nosim, resps = resps, designtype = destype,
      designpath = designpath, u = ul, bcoeff = "not a list"
    ))
  })

  test_that("B coefficients in the utility functions dont match those in the bcoeff list", {
    expect_error(sim_all(
      nosim = nosim, resps = resps, designtype = destype,
      designpath = designpath, u = ul, bcoeff = list(bWRONG = 0.00)
    ))
  })

  test_that("Utility functions are valid", {
    expect_no_error(eval(ul$u1$v1))
    expect_no_error(eval(ul$u1$v2))
  })

  test_that("Design path must be a valid directory", {
    # Test case: designpath is not a character string
    expect_error(sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = 123, u = ul, bcoeff = bcoeff))

    # Test case: designpath does not exist
    expect_error(sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = "/nonexistent/path", u = ul, bcoeff = bcoeff))

    # Test case: designpath is not a directory
    expect_error(sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = "path/to/a/file.txt", u = ul, bcoeff = bcoeff))
  })

  test_that("Resps must be an integer", {
    # Test case: resps is missing
    expect_error(sim_all(nosim = nosim, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff))

    # Test case: resps is not an integer
    expect_error(sim_all(nosim = nosim, resps = "abc", designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff))

    # Test case: resps is a numeric but not an integer
    expect_error(sim_all(nosim = nosim, resps = 1.5, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff))
  })

  test_that("Function exists in simulateDCE", {
    expect_true("sim_all" %in% ls("package:simulateDCE"))
  })

  test_that("Simulation results are reasonable", {
    result1 <- sim_all(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups)

    # obtain the names of the design files (without extensions)
    designs <- tools::file_path_sans_ext(list.files(designpath, full.names = FALSE))

    ## Now access the summary data. this is tricky because a different table is made for each design
    ## to address this, loop through all designs and run tests on each design
    for (design in designs) {
      # Access the summary data frame for the current design
      summaryTable <- result1[[design]][["summary"]]

      ## nested loop looks at each row starting with est_ (estimated bcoeffs)
      for (row_name in rownames(summaryTable)) {
        if (startsWith(row_name, "est_")) {
          betaCoeff <- sub("^est_", "", row_name) # simpkly strip prefix
          meanBeta <- summaryTable[row_name, "mean"]
          inputBeta <- as.integer(bcoeff[betaCoeff]) # access beta coefficient list by specific coefficient name

          # Perform tests for rows starting with "est_"
          expect_true(
            betaCoeff %in% names(bcoeff),
            sprintf("Variable est_%s does not exist in summary data frame", variable)
          )


          expect_gt(meanBeta, inputBeta - 1)
          expect_lt(meanBeta, inputBeta + 1)
        }
      }
    }
  })
}

###################
## FROM RBOOK #####
###################

designpath <- system.file("extdata", "Rbook", package = "simulateDCE")

# notes <- "This design consists of different heuristics. One group did not attend the methan attribute, another group only decided based on the payment"

notes <- "No Heuristics"

resps <- 40 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

# betacoefficients should not include "-"

bcoeff <- list(
  bsq = 0.00,
  bredkite = -0.05,
  bdistance = 0.50,
  bcost = -0.05,
  bfarm2 = 0.25,
  bfarm3 = 0.50,
  bheight2 = 0.25,
  bheight3 = 0.50
)

destype <- "spdesign"


# place your utility functions here
ul <- list(u1 = list(
  v1 = V.1 ~ bsq * alt1.sq,
  v2 = V.2 ~ bfarm2 * alt2.farm2 + bfarm3 * alt2.farm3 + bheight2 * alt2.height2 + bheight3 * alt2.height3 + bredkite * alt2.redkite + bdistance * alt2.distance + bcost * alt2.cost,
  v3 = V.3 ~ bfarm2 * alt3.farm2 + bfarm3 * alt3.farm3 + bheight2 * alt3.height2 + bheight3 * alt3.height3 + bredkite * alt3.redkite + bdistance * alt3.distance + bcost * alt3.cost
))

comprehensive_design_test(nosim = nosim, resps = resps, designtype = destype, designpath = designpath, ul = ul, bcoeff = bcoeff)

###############################
#### From feedadditives #######
###############################

designpath <- system.file("extdata", "feedadditives", package = "simulateDCE")

# notes <- "This design consists of different heuristics. One group did not attend the methan attribute, another group only decided based on the payment"

notes <- "Three heuristics"

resps <- 30 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)
destype <- "ngene"

# betacoefficients should not include "-"
bcoeff <- list(
  basc = 0.2,
  bcow = 0.3,
  badv = 0.3,
  bvet = 0.3,
  bfar = 0.3,
  bmet = 0.3,
  bbon = 0.3,
  bbon2 = 1.9,
  basc2 = 2
)

dgFeed <- c(0, 0.3, 0.6, 0.8, 1)

# place your utility functions here
ul <- list(
  u1 = list(
    v1 = V.1 ~ bcow * alt1.cow + badv * alt1.adv + bvet * alt1.vet + bfar * alt1.far + bmet * alt1.met + bbon * alt1.bon,
    v2 = V.2 ~ bcow * alt2.cow + badv * alt2.adv + bvet * alt2.vet + bfar * alt2.far + bmet * alt2.met + bbon * alt2.bon,
    v3 = V.3 ~ basc
  ),
  u2 = list(
    v1 = V.1 ~ bcow * alt1.cow + badv * alt1.adv + bvet * alt1.vet + bfar * alt1.far + bbon * alt1.bon,
    v2 = V.2 ~ bcow * alt2.cow + badv * alt2.adv + bvet * alt2.vet + bfar * alt2.far + bbon * alt2.bon,
    v3 = V.3 ~ basc
  ),
  u3 = list(
    v1 = V.1 ~ bbon2 * alt1.bon,
    v2 = V.2 ~ bbon2 * alt2.bon,
    v3 = V.3 ~ basc
  ),
  u4 = list(
    v1 = V.1 ~ basc2 + bcow * alt1.cow + badv * alt1.adv + bvet * alt1.vet + bfar * alt1.far + bmet * alt1.met + bbon * alt1.bon,
    v2 = V.2 ~ bcow * alt2.cow + badv * alt2.adv + bvet * alt2.vet + bfar * alt2.far + bmet * alt2.met + bbon * alt2.bon,
    v3 = V.3 ~ basc
  )
)

## comprehensive_design_test(nosim=nosim, resps=resps, destype=destype, designpath=designpath, ul = ul, bcoeff = bcoeff, decisiongroups = dgFeed)

devtools::load_all()

design_ngene <- system.file("extdata", "agora", "altscf_eff.ngd", package = "simulateDCE")

t0 <- readdesign(design_ngene)


design_sp <- system.file("extdata", "ValuGaps", "des1.RDS", package = "simulateDCE")





t <- readdesign(design_sp)
t2 <- readdesign(design_sp, designtype = "spdesign")
identical(t, t2)


design_idefix <- system.file("extdata", "Idefix_designs", "test_design2.RDS", package = "simulateDCE")

t3 <- readdesign(design_idefix)

c <- (readRDS(design_idefix))

---
title: "SE_drive-vignette"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{SE_drive-vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!-- There is an issue when creating these vignettes using usethis::use_vignette and 
devtools::build_vignettes() where the compilied vignette HTML files are placed in /doc
rather than inst/doc

Best Practice is to follow these steps
1. Create vignette using usethis::use_vignette("my-vignette")
2. After making changes, then run devtools::build_vignettes()
3. Rebuild using devtools::install(build_vignettes = TRUE)
4. Check that it is in the vignette environment using browseVigettes()

If vignette does not appear in gitHub, it is possibly due to a file heirarchy problem where rendered files 
appear in /doc instead of /inst/doc

To avoid this run:
tools::buildVignettes(dir = ".", tangle=TRUE)
dir.create("inst/doc")
file.copy(dir("vignettes", full.names=TRUE), "inst/doc", overwrite=TRUE)

More info here: https://community.rstudio.com/t/browsevignettes-mypackage-saying-no-vignettes-found/68656/7
-->

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
# Introduction

This vignette demonstrates how to use the `simulateDCE` package to simulate discrete choice experiments (DCEs). We will use a sample dataset and utility functions to generate simulated data and analyze the results.

```{r setup}
library(simulateDCE)
library(rlang)
library(formula.tools)
```

# Inititalize Variables

sim_all is the highest level function in the package and will run simulations for all designs contained in the specified design folder. Accordingly, this is generally the function the user will want to call. To prepare for using this function, a hypothesized utility function with corresponding beta coefficients representing the weight of each term must be declared in R like so:

```{r initialize}
# pass beta coefficients as a list
bcoeff <- list(
  bpreis = -0.01,
  blade = -0.07,
  bwarte = 0.02
)

manipulations <- list(
  alt1.x2 = expr(alt1.x2 / 10),
  alt1.x3 = expr(alt1.x3 / 10),
  alt2.x2 = expr(alt2.x2 / 10),
  alt2.x3 = expr(alt2.x3 / 10)
)


# place your utility functions here
ul <- list(
  u1 =

    list(
      v1 = V.1 ~ bpreis * alt1.x1 + blade * alt1.x2 + bwarte * alt1.x3,
      v2 = V.2 ~ bpreis * alt2.x1 + blade * alt2.x2 + bwarte * alt2.x3
    ),
  u2 = list(
    v1 = V.1 ~ bpreis * alt1.x1,
    v2 = V.2 ~ bpreis * alt2.x1
  )
)
```
# Decision Groups

If the utility function set passed to sim all has more than 1 utility function group as is the case above (u1, u2 etc.) an additional variable, 'decision groups' must be given to tell the function what portion of respondents should be given each utility function.

```{r decision}
decisiongroups <- c(0, 0.7, 1)
```

# Other parameters

Besides these arguments the user must also specify the number of respondents in the simulated survey and the number of times to run the model. The number of respondents (resps) should be selected based on experimental design parameters, while the number of simulations (nosim) should be large enough to glean statistically significant data. It is best to use a small number for this while learning to use the package and a large number (at least 500) once the other parameters have been settled.

The simulation can be ran using spdesign or NGENE design files which will be contained in the design path. The design path and design type, must also be specified: 

```{r other}
designpath <- system.file("extdata", "SE_DRIVE", package = "simulateDCE")

resps <- 120 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)


destype <- "ngene"
```

# Output

The sim_all function returns a multidimensional R list containing graphs, simulated observations and a dataframe containing sumaries of estimated b coefficients. In general these will be printed to the console, but the entire results can also be assigned to an r list object.

```{r output}
sedrive <- sim_all(
  nosim = nosim, resps = resps, designtype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, decisiongroups = decisiongroups
)
```

# Accessing the Output in R

The beta cofficients for each simulation are contained in a dataframe called coeffs located within {result}->olddesign->coefs. and a summary table is contained in ->olddesign->summary

```{r accessOutput}
## nested results are hard coded, if the design changes this must aswell
simulationCoeff <- sedrive$olddesign$coefs
coeffSummary <- sedrive$olddesign$summary

print(simulationCoeff)
print(coeffSummary)
```

---
title: "SE_Agri-vignette"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{SE_Agri-vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!-- There is an issue when creating these vignettes using usethis::use_vignette and 
devtools::build_vignettes() where the compilied vignette HTML files are placed in /doc
rather than inst/doc

Best Practice is to follow these steps
1. Create vignette using usethis::use_vignette("my-vignette")
2. After making changes, then run devtools::build_vignettes()
3. Rebuild using devtools::install(build_vignettes = TRUE)
4. Check that it is in the vignette environment using browseVigettes()

If vignette does not appear in gitHub, it is possibly due to a file heirarchy problem where rendered files 
appear in /doc instead of /inst/doc

To avoid this run:
tools::buildVignettes(dir = ".", tangle=TRUE)
dir.create("inst/doc")
file.copy(dir("vignettes", full.names=TRUE), "inst/doc", overwrite=TRUE)

More info here: https://community.rstudio.com/t/browsevignettes-mypackage-saying-no-vignettes-found/68656/7
-->

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

This vignette demonstrates how to use simulate situations in which different utility functions apply to diffferent subsets of respondents. We will use a sample dataset and utility functions to generate simulated data and analyze the results. First off it is good practice remove all objects (variables, functions, etc.) from the current workspace and  load all R files in the package directory into the current R session.

```{r setup}
library(simulateDCE)
library(rlang)
library(formula.tools)
```

# Inititalize Variables

sim_all is the highest level function in the package and will run simulations for all designs contained in the specified design folder. Accordingly, this is generally the function the user will want to call. To prepare for using this function, a hypothesized utility function with corresponding beta coefficients representing the weight of each term must be declared in R.

The manipulation variable allows the user to assign different terms based on the values of column values in the experimental design file

```{r initialize}
bcoeff <- list(
  basc = 4.2, ## very high asc
  bprof = 0.3,
  bexp = 0.3,
  bdomestic = 0.3,
  bforeign = 0.3,
  bdamage = 0.6,
  bprice = 0.2
)



manipulations <- list(
  alt1.professional = expr(alt1.initiator == 1),
  alt2.professional = expr(alt2.initiator == 1),
  alt1.expert = expr(alt1.initiator == 2),
  alt2.expert = expr(alt2.initiator == 2),
  alt1.domestic = expr(alt1.funding == 1),
  alt2.domestic = expr(alt2.funding == 1),
  alt1.foreign = expr(alt1.funding == 2),
  alt2.foreign = expr(alt2.funding == 2)
)


# place your utility functions here
ul <- list(
  u1 =
    list(
      v1 = V.1 ~ bprof * alt1.professional + bexp * alt1.expert + bdomestic * alt1.domestic + bforeign * alt1.foreign + bdamage * alt1.damage + bprice * alt1.compensation,
      v2 = V.2 ~ bprof * alt2.professional + bexp * alt2.expert + bdomestic * alt2.domestic + bforeign * alt2.foreign + bdamage * alt2.damage + bprice * alt2.compensation,
      v3 = V.3 ~ basc
    )
)
```
# Other parameters

Besides these arguments the user must also specify the number of respondents in the simulated survey and the number of times to run the model. The number of respondents (resps) should be selected based on experimental design parameters, while the number of simulations (nosim) should be large enough to glean statistically significant data. It is best to use a small number for this while learning to use the package and a large number (at least 500) once the other parameters have been settled.

The simulation can be ran using spdesign or NGENE design files which will be contained in the design path. The design path and design type, must also be specified as strings: 

```{r other}
designpath <- system.file("extdata", "SE_AGRI", package = "simulateDCE")
## can also be specified using relative path eg. designpath<- "Projects/CSA/Designs/"

# notes <- "This design consists of different heuristics. One group did not attend the methan attribute, another group only decided based on the payment"

notes <- "No Heuristics"

resps <- 240 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

## design type must be either 'spdesign' or 'ngene'
destype <- "ngene"
```
# Randomness

As several part of the simulation rely on random values within experimentally defined bounds, the output of a given simulation call using sim_all will vary each time it is called unless the seed for R's random number generator is set like so:

```{r random}
set.seed(3393)
```

# Output

The sim_all function returns a multidimensional R list containing graphs, simulated observations and a dataframe containing sumaries of estimated beta coefficients. In general these will be printed to the console, but the entire results can also be assigned to an r list object.

```{r output}
seagri <- simulateDCE::sim_all(
  nosim = nosim, resps = resps, designtype = destype,
  designpath = designpath, u = ul, bcoeff = bcoeff, manipulations = manipulations
)
```

# Accessing the Output in R

The beta cofficients for each simulation are contained in a dataframe called coeffs located within {result}->{design}->coefs. and a summary table, which displays statistics of these b coefficients across all simulations is contained in {results}->{design}->summary. You can access these by either searching for a known design name or by browsing design names contained in the output list.

You can also save the results to disk using saveRDS(csa,file = "tests/manual-tests/csa.RDS")


```{r accessOutput}
designs <- names(seagri[sapply(seagri, is.list)])

print(designs)

simulationCoeff <- seagri[[1]]$coefs
coeffSummary <- seagri[[1]]$summary

print(simulationCoeff)
print(coeffSummary)


## saveRDS(csa,file = "tests/manual-tests/csa.RDS")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregateResults.R
\name{aggregateResults}
\alias{aggregateResults}
\title{Aggregate Simulation Results}
\usage{
aggregateResults(all_designs, fromfolder = NULL)
}
\arguments{
\item{all_designs}{A list of simulation results from sim_choice. Can contain different designs but need to have the common structure returned by simchoice}

\item{fromfolder}{A folder from where to read simulations. If provided, the function will read all .qs files from the folder and process them. The files are usually saved by your earlier work and should be qs files as they are more efficient that rds files.}
}
\value{
A list with aggregated results including summary, coefficients, graphs, and power.
}
\description{
Processes the simulation results to extract summaries, coefficients, and graphs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readdesign.R
\name{readdesign}
\alias{readdesign}
\title{Creates a dataframe with the design.}
\usage{
readdesign(design = designfile, designtype = NULL, destype = NULL)
}
\arguments{
\item{design}{The path to a design file}

\item{designtype}{Is it a design created with ngene, spdesign or idefix. use 'ngene', 'spdesign' or 'idefix. Ngene designs should be stored as the standard .ngd output. spdesign should be the spdesign object stored as an RDS file. Idefix objects should also be stored as an RDS file. If designtype is not specified, I try to guess what it is. This is especially helpful if you want to carry out a simulation for both spdesign designs and ngene designs at the same time.}

\item{destype}{Deprecated. Use designtype instead.}
}
\value{
a dataframe
}
\description{
Creates a dataframe with the design.
}
\examples{
library(simulateDCE)
mydesign <- readdesign(
  system.file("extdata", "agora", "altscf_eff.ngd", package = "simulateDCE"),
  "ngene"
)

print(mydesign)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{modify_bcoeff_names}
\alias{modify_bcoeff_names}
\title{Modify bcoeff Names}
\usage{
modify_bcoeff_names(bcoeff)
}
\arguments{
\item{bcoeff}{A named list of beta coefficients. The names of this list are
the coefficients used in the utility function.}
}
\value{
A list containing:
\describe{
\item{bcoeff}{The modified \code{bcoeff} list with updated names.}
\item{bcoeff_lookup}{A tibble mapping the original names to the modified names.}
}
}
\description{
This function modifies the names of beta coefficients (\code{bcoeff}) by removing
dots and underscores to ensure consistent naming. If the names are already modified,
the function skips processing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_choice.R
\name{sim_choice}
\alias{sim_choice}
\title{Simulate and estimate choices}
\usage{
sim_choice(
  designfile,
  no_sim = 10,
  respondents = 330,
  u,
  designtype = NULL,
  destype = NULL,
  bcoeff,
  decisiongroups = c(0, 1),
  manipulations = list(),
  estimate,
  chunks = 1,
  utility_transform_type = "simple",
  mode = c("parallel", "sequential"),
  preprocess_function = NULL,
  savefile = NULL
)
}
\arguments{
\item{designfile}{path to a file containing a design.}

\item{no_sim}{Number of runs i.e. how often do you want the simulation to be repeated}

\item{respondents}{Number of respondents. How many respondents do you want to simulate in each run.}

\item{u}{A list with utility functions. The list can incorporate as many decision rule groups as you want. However, each group must be in a list in this list. If you just use one group (the normal),  this  group still  has to be in a list in  the u list. As a convention name beta coefficients starting with a lower case "b"}

\item{designtype}{Is it a design created with ngene, spdesign or idefix. use 'ngene', 'spdesign' or 'idefix. Ngene designs should be stored as the standard .ngd output. spdesign should be the spdesign object stored as an RDS file. Idefix objects should also be stored as an RDS file. If designtype is not specified, I try to guess what it is. This is especially helpful if you want to carry out a simulation for both spdesign designs and ngene designs at the same time.}

\item{destype}{Deprecated. Use designtype instead.}

\item{bcoeff}{List of initial coefficients for the utility function. List content/length can vary based on application. I ideally begins (but does not have to) with b and need be the same as those entered in the utility functions}

\item{decisiongroups}{A vector showing how decision groups are numerically distributed}

\item{manipulations}{A variable to alter terms of the utility functions examples may be applying a factor or applying changes to terms selectively for different groups}

\item{estimate}{If TRUE models will be estimated. If false only a dataset will be simulated. Default is TRUE}

\item{chunks}{The number of chunks determines how often results should be stored on disk as a safety measure to not loose simulations if models have already been estimated. For example, if no_sim is 100 and chunks = 2, the data will be saved on disk after 50 and after 100 runs.}

\item{utility_transform_type}{How the utility function you entered is transformed to the utility function required for mixl. You can use the classic way (simple) where parameters have to start with "b" and variables with "alt" or the more flexible (but potentially error prone) way (exact) where parameters and variables are matched exactly what how the are called in the dataset and in the bcoeff list. Default is "simple". In the long run, simple will be deleted, as exact should be downwards compatible.}

\item{mode}{Set to "parallel" if parts should be run in parallel mode}

\item{preprocess_function}{= NULL You can supply a function that reads in external data (e.g. GIS coordinates) that will be merged with the simulated dataset. Make sure the the function outputs a data.frame that has a variable called ID which is used for matching.}

\item{savefile}{Indicate a path if you want to store the results after each design simulation locally. This is useful in case you fear that your computer crashes}
}
\value{
a list with all information on the run
}
\description{
Simulate and estimate choices
}
\examples{
bcoeff <- list(
  basc = -1.2,
  basc2 = -1.4,
  baction = 0.1,
  badvisory = 0.4,
  bpartnertest = 0.3,
  bcomp = 0.02
)
ul <- list(
  u1 =
    list(
     #' # model specification ----------------------------------------------
v1 <- V.1 ~ basc +
  baction      * alt1.b +
  badvisory    * alt1.c +
  bpartnertest * alt1.d +
  bcomp        * alt1.p,

v2 <- V.2 ~ basc2 +
  baction      * alt2.b +
  badvisory    * alt2.c +
  bpartnertest * alt2.d +
  bcomp        * alt2.p,
 v3 <- V.3 ~ 0
    )
)

sim_choice(
  designfile = system.file("extdata", "agora", "altscf_eff.ngd", package = "simulateDCE"),
  no_sim = 2,
  respondents = 144,
  u = ul,
  bcoeff = bcoeff,
  estimate = FALSE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDataset.R
\name{createDataset}
\alias{createDataset}
\title{Create a Dataset for Choice Experiment Analysis}
\usage{
createDataset(design, respondents)
}
\arguments{
\item{design}{A data frame containing the design matrix for the choice experiment.
It should include at least the columns \code{Choice.situation} and optionally \code{Block}.}

\item{respondents}{The number of respondents to generate data for.}
}
\value{
A data frame containing the augmented design matrix with additional columns:
\describe{
\item{ID}{A unique identifier for each respondent.}
\item{Choice.situation}{The original choice situations, replicated for respondents.}
\item{Other columns}{All original columns in the input \code{design} are retained.}
}
}
\description{
This function takes a design matrix and generates a dataset for use in choice experiments.
It handles blocks, replicates the design for the number of respondents, and assigns respondent IDs.
}
\details{
The function performs the following steps:
\itemize{
\item Checks if the \code{Block} column exists in the input design. If absent, it creates a single block.
\item Calculates the number of choice sets and blocks, and determines the number of sets per block.
\item Replicates the design to account for the specified number of respondents per block.
\item Assigns respondent IDs based on the number of respondents and blocks.
}
}
\examples{
# Example usage:
design <- data.frame(
  Choice.situation = rep(1:12),
  Attribute1 = rnorm(12),
  Attribute2 = sample(1:3, 12, replace = TRUE)
)
result <- createDataset(design, 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_all.R
\name{sim_all}
\alias{sim_all}
\title{Is a wrapper for sim_choice executing the simulation over all designs stored in a specific folder
update}
\usage{
sim_all(
  nosim = 2,
  resps,
  designtype = NULL,
  destype = NULL,
  designpath,
  u,
  bcoeff,
  decisiongroups = c(0, 1),
  manipulations = list(),
  estimate = TRUE,
  chunks = 1,
  utility_transform_type = "simple",
  reshape_type = "auto",
  mode = c("parallel", "sequential"),
  preprocess_function = NULL,
  savefile = NULL
)
}
\arguments{
\item{nosim}{Number of runs or simulations. For testing use 2 but once you go serious, use at least 200, for better results use 2000.}

\item{resps}{Number of respondents you want to simulate}

\item{designtype}{Is it a design created with ngene, spdesign or idefix. use 'ngene', 'spdesign' or 'idefix. Ngene designs should be stored as the standard .ngd output. spdesign should be the spdesign object stored as an RDS file. Idefix objects should also be stored as an RDS file. If designtype is not specified, I try to guess what it is. This is especially helpful if you want to carry out a simulation for both spdesign designs and ngene designs at the same time.}

\item{destype}{Deprecated. Use designtype instead.}

\item{designpath}{The path to the folder where the designs are stored. For example "c:/myfancydec/Designs"}

\item{u}{A list with utility functions. The list can incorporate as many decision rule groups as you want. However, each group must be in a list in this list. If you just use one group (the normal),  this  group still  has to be in a list in  the u list. As a convention name beta coefficients starting with a lower case "b"}

\item{bcoeff}{List of initial coefficients for the utility function. List content/length can vary based on application. I ideally begins (but does not have to) with b and need be the same as those entered in the utility functions}

\item{decisiongroups}{A vector showing how decision groups are numerically distributed}

\item{manipulations}{A variable to alter terms of the utility functions examples may be applying a factor or applying changes to terms selectively for different groups}

\item{estimate}{If TRUE models will be estimated. If false only a dataset will be simulated. Default is TRUE}

\item{chunks}{The number of chunks determines how often results should be stored on disk as a safety measure to not loose simulations if models have already been estimated. For example, if no_sim is 100 and chunks = 2, the data will be saved on disk after 50 and after 100 runs.}

\item{utility_transform_type}{How the utility function you entered is transformed to the utility function required for mixl. You can use the classic way (simple) where parameters have to start with "b" and variables with "alt" or the more flexible (but potentially error prone) way (exact) where parameters and variables are matched exactly what how the are called in the dataset and in the bcoeff list. Default is "simple". In the long run, simple will be deleted, as exact should be downwards compatible.}

\item{reshape_type}{Must be "auto", "stats" to use the reshape from the stats package or tidyr to use pivot longer. Default is auto and should not bother you. Only change it once you face an error at this position and you may be lucky that it works then.}

\item{mode}{Set to "parallel" if parts should be run in parallel mode}

\item{preprocess_function}{= NULL You can supply a function that reads in external data (e.g. GIS coordinates) that will be merged with the simulated dataset. Make sure the the function outputs a data.frame that has a variable called ID which is used for matching.}

\item{savefile}{Indicate a path if you want to store the results after each design simulation locally. This is useful in case you fear that your computer crashes}
}
\value{
A list, with all information on the simulation. This list an be easily processed by the user and in the rmarkdown template.
}
\description{
Is a wrapper for sim_choice executing the simulation over all designs stored in a specific folder
update
}
\examples{
library(rlang)
designpath <- system.file("extdata", "SE_DRIVE", package = "simulateDCE")
resps <- 120 # number of respondents
nosim <- 2 # number of simulations to run (about 500 is minimum)

decisiongroups <- c(0, 0.7, 1)

# pass beta coefficients as a list
bcoeff <- list(
  b.preis = -0.01,
  b.lade = -0.07,
  b.warte = 0.02
)

manipulations <- list(
  alt1.x2 = expr(alt1.x2 / 10),
  alt1.x3 = expr(alt1.x3 / 10),
  alt2.x2 = expr(alt2.x2 / 10),
  alt2.x3 = expr(alt2.x3 / 10)
)


# place your utility functions here
ul <- list(
  u1 =

    list(
      v1 = V.1 ~ b.preis * alt1.x1 + b.lade * alt1.x2 + b.warte * alt1.x3,
      v2 = V.2 ~ b.preis * alt2.x1 + b.lade * alt2.x2 + b.warte * alt2.x3
    ),
  u2 = list(
    v1 = V.1 ~ b.preis * alt1.x1,
    v2 = V.2 ~ b.preis * alt2.x1
  )
)


sedrive <- sim_all(
  nosim = nosim,
  resps = resps,
  designpath = designpath,
  u = ul,
  bcoeff = bcoeff,
  decisiongroups = decisiongroups,
  manipulations = manipulations,
  utility_transform_type = "exact",
  mode = "sequential",
  estimate=FALSE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spdesign_utils.R
\name{extract_b_values}
\alias{extract_b_values}
\title{Title Extracts beta values from an spdesign object}
\usage{
extract_b_values(input_list)
}
\arguments{
\item{input_list}{the list where the parameters are stored. Usually this is \code{design$utility}}
}
\value{
A named list with parameter values which can be used in \code{sim_all}
}
\description{
Title Extracts beta values from an spdesign object
}
\examples{
d <- system.file("extdata", "CSA", "linear", "BLIeff.RDS", package = "simulateDCE")
extract_b_values(readRDS(d)$utility)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_choices.R
\name{simulate_choices}
\alias{simulate_choices}
\title{Simulate choices based on a data.frame with a design and respondents}
\usage{
simulate_choices(
  data,
  utility,
  setspp,
  bcoeff,
  decisiongroups = c(0, 1),
  manipulations = list(),
  preprocess_function = NULL
)
}
\arguments{
\item{data}{a dataframe that includes a design repeated for the number of observations}

\item{utility}{a list with the utility functions, one utility function for each alternatives}

\item{setspp}{Deprecated. Ignored and retained for backward compatibility.}

\item{bcoeff}{List of initial coefficients for the utility function. List content/length can vary based on application. I ideally begins (but does not have to) with b and need be the same as those entered in the utility functions}

\item{decisiongroups}{A vector showing how decision groups are numerically distributed}

\item{manipulations}{A variable to alter terms of the utility functions examples may be applying a factor or applying changes to terms selectively for different groups}

\item{preprocess_function}{= NULL You can supply a function that reads in external data (e.g. GIS coordinates) that will be merged with the simulated dataset. Make sure the the function outputs a data.frame that has a variable called ID which is used for matching.}
}
\value{
a data.frame that includes simulated choices and a design
}
\description{
Simulate choices based on a data.frame with a design and respondents
}
\examples{
example_df <- data.frame(
  ID = rep(1:100, each = 4),
  price = rep(c(10, 10, 20, 20), 100),
  quality = rep(c(1, 2, 1, 2), 100)
)

beta <- list(
  bprice   = -0.2,
  bquality =  0.8
)

ut <- list(
  u1 = list(
    v1 = V.1 ~ bprice * price + bquality * quality,
    v2 = V.2 ~ 0
  )
)
simulate_choices(example_df, ut, setspp = 4, bcoeff = beta)

}
